//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: CacheDatabase.cpp,v 1.13 2008/03/18 20:52:53 gmf Exp $
//

#include "DbXmlInternal.hpp"
#include "CacheDatabase.hpp"
#include "Cursor.hpp"
#include "nodeStore/NsFormat.hpp"

using namespace std;
using namespace DbXml;

#define DEFAULT_CACHE_PAGESIZE (8 * 1024)

// TBD Should we specify a page size here, or just leave it at 0 for the default - jpcs
CacheDatabase::CacheDatabase(DbEnv *env, DBTYPE type,
			     bt_compare_fcn_type compare)
	: db_(env, "", "", "", DEFAULT_CACHE_PAGESIZE, 0)
{
	if (compare) {
		DBXML_ASSERT(type == DB_BTREE);
		db_.getDb().set_bt_compare(compare);
	}
	int err = db_.open(0, type, DB_CREATE|DB_NO_AUTO_COMMIT, 0);
	db_.setReadCursor(new Cursor(db_, NULL, CURSOR_READ));
	db_.setWriteCursor(new Cursor(db_, NULL, CURSOR_WRITE));
	if(err != 0) throw XmlException(err);
}

DbEnv *CacheDatabase::getDbEnv()
{
	return db_.getEnvironment();
}

RecordDatabase::RecordDatabase(DbEnv *env) : CacheDatabase(env, DB_RECNO, 0)
{
}

DocDatabase::DocDatabase(DbEnv *env,
			 bt_compare_fcn_type compare)
	: CacheDatabase(env, DB_BTREE, compare)
{
}

//static
bool CacheDatabaseHandle::docExists(DbWrapper *db, const DocID &id)
{
	DbtOut key;
	DbtIn none(0, 0);
	none.set_flags(DB_DBT_PARTIAL); // Don't pull back the data.

	// Use the doc metadata node to test for existence.
	NsFormat::marshalNodeKey(id, *NsNid::getMetaDataNid(), key);
	int err = db->get(0, &key, &none, 0);
	return (err == 0);
}

