//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: CacheDatabase.hpp,v 1.8 2008/01/18 03:52:24 gmf Exp $
//

#ifndef __CACHEDATABASE_HPP
#define	__CACHEDATABASE_HPP

#include <string>
#include <db_cxx.h>
#include <errno.h>
#include "dbxml/XmlException.hpp"
#include "DbWrapper.hpp"
#include "ReferenceCounted.hpp"

namespace DbXml
{

class OperationContext;
class DocID;
	
class CacheDatabase : public ReferenceCounted {
public:
	CacheDatabase(DbEnv *env, DBTYPE type, bt_compare_fcn_type compare);
	CacheDatabase(): db_(0) {}
	virtual ~CacheDatabase() {}
	DbEnv *getDbEnv();
	DbWrapper *getDb() { return &db_; }

protected:
	DbWrapper db_;
};

class RecordDatabase : public CacheDatabase {
public:
	RecordDatabase(DbEnv *env);
};

class DocDatabase : public CacheDatabase {
public:
	DocDatabase() {}
	DocDatabase(DbEnv *env, bt_compare_fcn_type compare = 0);
};


class CacheDatabaseHandle
{
public:
	CacheDatabaseHandle() : cdb_(0) {}
	CacheDatabaseHandle(CacheDatabase *cdb) : cdb_(cdb)
	{
		if (cdb_ != 0)
			cdb_->acquire();
	}
	~CacheDatabaseHandle()
	{
		if (cdb_ != 0)
			cdb_->release();
	}

	operator CacheDatabase &() const {
		return *cdb_;
	}
	operator CacheDatabase *() const {
		return cdb_;
	}

	DbWrapper *getDb() const {
		if (cdb_) return cdb_->getDb();
		return 0;
	}

	CacheDatabaseHandle &operator=(const CacheDatabaseHandle &o)
	{
		if (this != &o && cdb_ != o.cdb_) {
			if (cdb_ != 0)
				cdb_->release();
			cdb_ = o.cdb_;
			if (cdb_ != 0)
				cdb_->acquire();
		}
		return *this;
	}
	CacheDatabaseHandle &operator=(CacheDatabase *cdb)
	{
		if (cdb_ != cdb) {
			if (cdb_ != 0)
				cdb_->release();
			cdb_ = cdb;
			if (cdb_ != 0)
				cdb_->acquire();
		}
		return *this;
	}
	bool isNull() const { return (cdb_ == 0); }
	static bool docExists(DbWrapper *db, const DocID &did);
private:
	CacheDatabase *cdb_;
};
	
}

#endif

