//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: CacheDatabaseMinder.cpp,v 1.7 2008/01/08 21:54:22 bostic Exp $
//

#include "DbXmlInternal.hpp"
#include "CacheDatabaseMinder.hpp"
#include "Manager.hpp"

using namespace DbXml;

CacheDatabase *CacheDatabaseMinder::findOrAllocate(Manager &mgr, int cid,
						   bool allocate)
{
	if (allocate)
		init(mgr);
	if (impl_)
		return impl_->findOrAllocate(cid, allocate);
	return 0;
}

 // allocate an implementation if not present
void CacheDatabaseMinder::init(Manager &mgr)
{
	if (!impl_) {
		impl_ = new CacheDatabaseMinderImpl(mgr);
		if (!impl_)
			throw XmlException(XmlException::NO_MEMORY_ERROR,
					   "Unable to allocate memory for object");
		impl_->acquire();
	}
}

CacheDatabaseMinderImpl::~CacheDatabaseMinderImpl()
{
	size_t size = entries_.size();
	for (size_t i = 0; i < size; i++) {
		if (entries_[i]) {
			delete entries_[i];
			entries_[i] = 0; // not necessary, but..
		}
	}
}
	
// find or create a cache db for container id
CacheDatabase *CacheDatabaseMinderImpl::findOrAllocate(int cid,
						       bool allocate)
{
	if (cid == 0)
		return verify(tempDb_, cid);
	size_t size = entries_.size();
	size_t i;
	for (i = 0; i < size; i++) {
		if (entries_[i]) {
		    if (entries_[i]->getContainerID() == cid) {
			    DBXML_ASSERT(entries_[i]->getDB());
			    return entries_[i]->getDB();
		    }
		} else
			break;
	}
	if (!allocate)
		return 0;
	if (i == size)
		entries_.resize(size << 1);
	CacheDBEntry *newentry = new CacheDBEntry();
	entries_[i] = newentry;
	return verify(*newentry, cid);
}

CacheDatabase *CacheDatabaseMinderImpl::verify(CacheDBEntry &entry, int cid)
{
	if (entry.getDB() == 0) {
		entry.setDB(mgr_.createDocDatabase());
		entry.setContainerID(cid);
	}
	DBXML_ASSERT(entry.getDB() != 0);
	return entry.getDB();
}
