//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: DictionaryCache.hpp,v 1.4 2008/01/08 21:54:23 bostic Exp $
//

#ifndef __DICTIONARYCACHE_HPP
#define	__DICTIONARYCACHE_HPP

#include "DbXmlInternal.hpp"
#include "NameID.hpp"
#include "ReferenceCounted.hpp"

namespace DbXml
{

class DictionaryCacheEntry;
class DictionaryCacheBuffer;
class DictionaryDatabase;
class OperationContext;
	
/**
 * Implements a hash table cache of ID->name for a dictionary
 */
class DictionaryCache
{
public:
	DictionaryCache();
	~DictionaryCache();
	void setDictionaryDatabase(DictionaryDatabase *ddb) {
		ddb_ = ddb;
	}
	// Only one basic lookup, returning a pointer that will be valid
	// until this object is deleted
	const char *lookup(OperationContext &context, const NameID &id,
			   bool useDictionary);
	bool lookup(OperationContext &context, const NameID &id,
		    DbtOut &dbt, bool useDictionary);
	void insert(const nameId_t nid, const char *value);
	void insert(const nameId_t nid, const Dbt &value);

private:
	static int hash(const nameId_t nid) {
		return (nid % hashSize_);
	}
private:
	DictionaryDatabase *ddb_;
	DictionaryCacheEntry **htable_;
	DictionaryCacheBuffer *mem_;
	dbxml_mutex_t mutex_;
	// configuration
	static int hashSize_;
};

}

#endif

