//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: MutexLock.cpp,v 1.4 2008/01/15 22:30:30 gmf Exp $
//

#include "DbXmlInternal.hpp"
#include "MutexLock.hpp"
#include "Globals.hpp"
#include <xercesc/util/PlatformUtils.hpp>

#if defined(XERCES_HAS_CPP_NAMESPACE)
  XERCES_CPP_NAMESPACE_USE
#endif

using namespace DbXml;

// it is safe to pass a NULL for mutex in which case
// this class is a big no-op.  This allows conditional
// locks
MutexLock::MutexLock(dbxml_mutex_t mutex)
	: toLock_(mutex)
{
	if (!MUTEX_IS_NULL(toLock_))
		XMLPlatformUtils::lockMutex(toLock_);
}

MutexLock::~MutexLock()
{
	if (!MUTEX_IS_NULL(toLock_))
		XMLPlatformUtils::unlockMutex(toLock_);
}

void MutexLock::unlock()
{
	if (!MUTEX_IS_NULL(toLock_))
		XMLPlatformUtils::unlockMutex(toLock_);
}

void MutexLock::lock()
{
	if (!MUTEX_IS_NULL(toLock_))
		XMLPlatformUtils::lockMutex(toLock_);
}

// static
dbxml_mutex_t MutexLock::createMutex()
{
	return (dbxml_mutex_t) XMLPlatformUtils::makeMutex(
		Globals::defaultMemoryManager);
}

// static
void MutexLock::destroyMutex(dbxml_mutex_t mutex)
{
	XMLPlatformUtils::closeMutex(mutex);
}

// static
void MutexLock::unlockMutex(dbxml_mutex_t mutex)
{
	XMLPlatformUtils::unlockMutex(mutex);
}

// static
void MutexLock::lockMutex(dbxml_mutex_t mutex)
{
	XMLPlatformUtils::lockMutex(mutex);
}
