//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: DbXmlASTNode.hpp,v 1.12 2008/01/08 21:54:23 bostic Exp $
//

#ifndef __DBXMLASTNODE_HPP
#define	__DBXMLASTNODE_HPP

#include "DbXmlResult.hpp"

#include <xqilla/ast/ASTNodeImpl.hpp>

namespace DbXml
{

class DbXmlASTNode : public ASTNodeImpl
{
public:
	enum whichType {
		NODE_CHECK = ASTNode::DIRECT_NAME + 20,
		LAST_STEP_CHECK,
		MAP,
		DBXML_PREDICATE,
		QP_TO_AST
	};

	static DbXmlASTNode *getDbXmlASTNode(ASTNode *ast)
	{
		switch(ast->getType()) {
		case ((ASTNode::whichType)NODE_CHECK):
		case ((ASTNode::whichType)LAST_STEP_CHECK):
		case ((ASTNode::whichType)MAP):
		case ((ASTNode::whichType)DBXML_PREDICATE):
		case ((ASTNode::whichType)QP_TO_AST):
			return (DbXmlASTNode*)ast;
		default: break;
		}
		return 0;
	}

	DbXmlASTNode(whichType type, XPath2MemoryManager* memMgr)
		: ASTNodeImpl(memMgr)
	{
		setType((ASTNode::whichType)type);
	}

	virtual ~DbXmlASTNode() {}

	/// This has been called by the time any DbXmlASTNode
	/// objects are created, and won't be called again.
	virtual ASTNode *staticResolution(StaticContext *context)
	{
		return this;
	}

	virtual Result createResult(DynamicContext *context, int flags=0) const
	{
		static const unsigned int props =
			StaticAnalysis::DOCORDER | StaticAnalysis::GROUPED |
			StaticAnalysis::PEER | StaticAnalysis::SUBTREE |
			StaticAnalysis::SAMEDOC | StaticAnalysis::ONENODE;
		return createResult(0, props, context);
	}

	virtual DbXmlResult createDbXmlResult(DynamicContext* context) const
	{
		static const unsigned int props =
			StaticAnalysis::DOCORDER | StaticAnalysis::GROUPED |
			StaticAnalysis::PEER | StaticAnalysis::SUBTREE |
			StaticAnalysis::SAMEDOC | StaticAnalysis::ONENODE;
		return createResult(0, props, context);
	}

	virtual DbXmlResult createResult(const DbXmlResult &contextItems, unsigned int props,
		DynamicContext *context) const = 0;

	static DbXmlResult createDbXmlResult(ASTNode *ast, DynamicContext *context)
	{
		DbXmlASTNode *da = getDbXmlASTNode(ast);
		if(da != 0) return da->createDbXmlResult(context);
		return ast->createResult(context).get();
	}
};

}

#endif
