//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: Map.hpp,v 1.6 2008/01/08 21:54:23 bostic Exp $
//

#ifndef __MAP_HPP
#define	__MAP_HPP

#include "DbXmlNodeImpl.hpp"
#include "DbXmlASTNode.hpp"
#include "../IndexEntry.hpp"

#include <xqilla/context/VariableStore.hpp>

namespace DbXml
{

class Map : public DbXmlASTNode
{
public:
	Map(ASTNode *arg1, ASTNode *arg2, const XMLCh *uri, const XMLCh *name, XPath2MemoryManager* memMgr);

	ASTNode *getArg1() const { return arg1_; }
	void setArg1(ASTNode *a) { arg1_ = a; }
	ASTNode *getArg2() const { return arg2_; }
	void setArg2(ASTNode *a) { arg2_ = a; }

	const XMLCh *getURI() const { return uri_; }
	const XMLCh *getName() const { return name_; }

	virtual ASTNode *staticTyping(StaticContext *context);
	virtual DbXmlResult createResult(const DbXmlResult &contextItems, unsigned int props,
		DynamicContext *context) const;

private:
	ASTNode *arg1_;
	ASTNode *arg2_;

	const XMLCh *uri_;
	const XMLCh *name_;
	StaticAnalysis varSrc_;
};

class MapResult : public ResultImpl, public VariableStore
{
public:
	MapResult(const Result &parent, const ASTNode *step, const XMLCh *uri, const XMLCh *name);

	virtual Result getVar(const XMLCh *namespaceURI, const XMLCh *name) const;

	Item::Ptr next(DynamicContext *context);
	std::string asString(DynamicContext *context, int indent) const { return ""; }

private:
	Result parent_;
	const ASTNode *step_;
	const XMLCh *uri_;
	const XMLCh *name_;

	Result stepResult_;

	Item::Ptr item_;
	const VariableStore *scope_;
};

class DbXmlPredicate : public DbXmlASTNode
{
public:
	DbXmlPredicate(ASTNode *expr, ASTNode *predicate, const XMLCh *uri, const XMLCh *name, XPath2MemoryManager *mm);

	const ASTNode *getExpression() const { return expr_; }
	void setExpression(ASTNode *expr) { expr_ = expr; }

	const ASTNode *getPredicate() const { return pred_; }
	void setPredicate(ASTNode *pred) { pred_ = pred; }

	const XMLCh *getURI() const { return uri_; }
	const XMLCh *getName() const { return name_; }

	virtual ASTNode *staticTyping(StaticContext *context);
	virtual DbXmlResult createResult(const DbXmlResult &contextItems, unsigned int props,
		DynamicContext *context) const;

protected:
	ASTNode *expr_;
	ASTNode *pred_;

	const XMLCh *uri_;
	const XMLCh *name_;
	StaticAnalysis varSrc_;
};

class VarPredicateResult : public ResultImpl, public VariableStore
{
public:
	VarPredicateResult(const Result &parent, const ASTNode *pred, const XMLCh *uri, const XMLCh *name);

	virtual Result getVar(const XMLCh *namespaceURI, const XMLCh *name) const;

	Item::Ptr next(DynamicContext *context);
	std::string asString(DynamicContext *context, int indent) const { return ""; }

private:
	Result parent_;
	const ASTNode *pred_;
	const XMLCh *uri_;
	const XMLCh *name_;

	Item::Ptr item_;
	const VariableStore *scope_;
};

class DbXmlNodeCheck : public DbXmlASTNode
{
public:
	DbXmlNodeCheck(ASTNode *arg, XPath2MemoryManager* memMgr);

	ASTNode *getArg() const { return arg_; }
	void setArg(ASTNode *a) { arg_ = a; }

	virtual ASTNode *staticTyping(StaticContext *context);
	virtual DbXmlResult createResult(const DbXmlResult &contextItems, unsigned int props,
		DynamicContext *context) const;

private:
	ASTNode *arg_;
};

class DbXmlLastStepCheck : public DbXmlASTNode
{
public:
	DbXmlLastStepCheck(ASTNode *arg, XPath2MemoryManager* memMgr);

	ASTNode *getArg() const { return arg_; }
	void setArg(ASTNode *a) { arg_ = a; }

	virtual ASTNode *staticTyping(StaticContext *context);
	virtual DbXmlResult createResult(const DbXmlResult &contextItems, unsigned int props,
		DynamicContext *context) const;

private:
	ASTNode *arg_;
};


}

#endif
