//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: NsHandlerBase.hpp,v 1.26 2008/01/08 21:54:24 bostic Exp $
//

#ifndef __DBXMLNSHANDLERBASE_HPP
#define __DBXMLNSHANDLERBASE_HPP

/*
 * NsHandlerBase
 *
 * A base class for handling NsEvent input, used for
 * creation of Node Storage nodes, as well as
 * "transient" parsing for whole-document storage
 * input and querying.
 *
 * It has no virtual behavior.
 */

#include "NsTypes.hpp"
#include "NsUtil.hpp"
#include "NsDocument.hpp"
#include "NsNid.hpp"

namespace DbXml
{

class EventWriter;
struct nsTextList;
	
class NsHandlerBase {
public:
	NsHandlerBase(NsDocument *doc,
		      EventWriter *writer);
	~NsHandlerBase();

	// Configuration
	void setBaseEventWriter(EventWriter *writer) {
		_ewriter = writer;
	}
	EventWriter *getBaseEventWriter() const {
		return _ewriter;
	}

	// Explicit end call for partial documents
	void end(void);

	void nextId(NsFullNid *id);
	void incDepth() { ++_depth; }
	void decDepth() { --_depth; }

protected:
	void addText(const xmlbyte_t *text, size_t len, uint32_t textType,
		     bool isDonated = false);
	void addPI(const xmlbyte_t *target, const xmlbyte_t *data,
		   bool isDonated, size_t len);
	void startElem(NsNode *node,
		       const xmlbyte_t *baseName,
		       const char *uri, // may be null
		       const char *prefix,
		       bool isDonated);
	void endDoc(void);
	void endElem(void);
protected:
	// parse state
	NsDocument *_doc;
	EventWriter *_ewriter;
	NsNode* _current;
	NsNode* _previous;
	uint32_t _depth;
	bool _lastWasText;
	struct nsTextList *_textList;

	// nid
	NsNidGen _nidGen;
};

}

#endif
