//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: ImpliedSchemaGenerator.cpp,v 1.23 2008/03/12 15:00:00 jpcs Exp $
//

#include "../DbXmlInternal.hpp"
#include "ImpliedSchemaGenerator.hpp"
#include "../QueryContext.hpp"
#include "../Value.hpp"
#include "../ContainerBase.hpp"
#include "../Manager.hpp"
#include "../dataItem/DbXmlDocAvailable.hpp"
#include "../dataItem/MetaDataFunction.hpp"
#include "../dataItem/DbXmlNodeTest.hpp"
#include "../dataItem/DbXmlUserData.hpp"
#include "../dataItem/LookupIndexFunction.hpp"
#include "../dataItem/DbXmlContains.hpp"
#include "../dataItem/DbXmlConfiguration.hpp"
#include "../dataItem/NodeHandleFunction.hpp"

#include <xqilla/context/DynamicContext.hpp>

#include <xqilla/operators/And.hpp>
#include <xqilla/operators/Or.hpp>
#include <xqilla/operators/Equals.hpp>
#include <xqilla/operators/NotEquals.hpp>
#include <xqilla/operators/Plus.hpp>
#include <xqilla/operators/Minus.hpp>
#include <xqilla/operators/Multiply.hpp>
#include <xqilla/operators/Divide.hpp>
#include <xqilla/operators/IntegerDivide.hpp>
#include <xqilla/operators/Mod.hpp>
#include <xqilla/operators/LessThan.hpp>
#include <xqilla/operators/GreaterThan.hpp>
#include <xqilla/operators/LessThanEqual.hpp>
#include <xqilla/operators/GreaterThanEqual.hpp>
#include <xqilla/operators/UnaryMinus.hpp>
#include <xqilla/operators/NodeComparison.hpp>
#include <xqilla/operators/OrderComparison.hpp>
#include <xqilla/operators/GeneralComp.hpp>
#include <xqilla/operators/Range.hpp>
#include <xqilla/operators/Intersect.hpp>
#include <xqilla/operators/Except.hpp>
#include <xqilla/operators/Union.hpp>

#include <xqilla/functions/FunctionBoolean.hpp>
#include <xqilla/functions/FunctionBaseURI.hpp>
#include <xqilla/functions/FunctionCollection.hpp>
#include <xqilla/functions/FunctionContains.hpp>
#include <xqilla/functions/FunctionCount.hpp>
#include <xqilla/functions/FunctionData.hpp>
#include <xqilla/functions/FunctionDeepEqual.hpp>
#include <xqilla/functions/FunctionDistinctValues.hpp>
#include <xqilla/functions/FunctionDoc.hpp>
#include <xqilla/functions/FunctionEmpty.hpp>
#include <xqilla/functions/FunctionEndsWith.hpp>
#include <xqilla/functions/FunctionExactlyOne.hpp>
#include <xqilla/functions/FunctionExists.hpp>
#include <xqilla/functions/FunctionInScopePrefixes.hpp>
#include <xqilla/functions/FunctionNamespaceURIForPrefix.hpp>
#include <xqilla/functions/FunctionNilled.hpp>
#include <xqilla/functions/FunctionId.hpp>
#include <xqilla/functions/FunctionIdref.hpp>
#include <xqilla/functions/FunctionInsertBefore.hpp>
#include <xqilla/functions/FunctionLang.hpp>
#include <xqilla/functions/FunctionLast.hpp>
#include <xqilla/functions/FunctionLocalname.hpp>
#include <xqilla/functions/FunctionMax.hpp>
#include <xqilla/functions/FunctionMin.hpp>
#include <xqilla/functions/FunctionName.hpp>
#include <xqilla/functions/FunctionNamespaceUri.hpp>
#include <xqilla/functions/FunctionNormalizeSpace.hpp>
#include <xqilla/functions/FunctionNot.hpp>
#include <xqilla/functions/FunctionNumber.hpp>
#include <xqilla/functions/FunctionOneOrMore.hpp>
#include <xqilla/functions/FunctionParseXML.hpp>
#include <xqilla/functions/FunctionParseJSON.hpp>
#include <xqilla/functions/FunctionPosition.hpp>
#include <xqilla/functions/FunctionRemove.hpp>
#include <xqilla/functions/FunctionReverse.hpp>
#include <xqilla/functions/FunctionRoot.hpp>
#include <xqilla/functions/FunctionStartsWith.hpp>
#include <xqilla/functions/FunctionString.hpp>
#include <xqilla/functions/FunctionStringLength.hpp>
#include <xqilla/functions/FunctionSubsequence.hpp>
#include <xqilla/functions/FunctionTrace.hpp>
#include <xqilla/functions/FunctionUnordered.hpp>
#include <xqilla/functions/FunctionZeroOrOne.hpp>

#if defined(XERCES_HAS_CPP_NAMESPACE)
XERCES_CPP_NAMESPACE_USE
#endif

using namespace DbXml;
using namespace std;

static const XMLCh XMLChDot[] = { chColon, chColon, chLatin_d, chLatin_o, chLatin_t, chNull };

ImpliedSchemaGenerator::ImpliedSchemaGenerator(DynamicContext *xpc, Optimizer *parent)
	: Optimizer(parent),
	  xpc_(xpc),
	  varStore_(&memMgr_)
{
	resetInternal();
}

void ImpliedSchemaGenerator::createAnyNodeResult(PathResult &result)
{
	XPath2MemoryManager *mm = xpc_->getMemoryManager();

	DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::document_string);
        ImpliedSchemaNode *root = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, mm);
	roots_.push_back(root);
	result.join(root);

	ImpliedSchemaNode *pisn = root->appendChild(new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(0),
							    ImpliedSchemaNode::DESCENDANT, mm));
	result.join(pisn);

	ImpliedSchemaNode *cisn = pisn->appendChild(new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(Node::attribute_string),
							    ImpliedSchemaNode::ATTRIBUTE, mm));
	result.join(cisn);
}

void ImpliedSchemaGenerator::resetInternal()
{
	varStore_.clear();
	roots_.clear();

	// The initial context item could be a node of any type from anywhere -
	// set up a set of paths that reflect that

	PathResult contextResult;
	createAnyNodeResult(contextResult);

	varStore_.setGlobalVar(XMLChDot, XMLChDot, contextResult);
}

ASTNode *ImpliedSchemaGenerator::optimize(ASTNode *item)
{
	generate(item).markSubtreeResult();
	return item;
}

void ImpliedSchemaGenerator::optimize(XQQuery *query)
{
	// Generate the imported module's global variables
	ImportedModules &modules = const_cast<ImportedModules&>(query->getImportedModules());
	for(ImportedModules::const_iterator it2 = modules.begin(); it2 != modules.end(); ++it2) {
		optimize(*it2);
	}

	for(UserFunctions::const_iterator it = query->getFunctions().begin();
	    it != query->getFunctions().end(); ++it) {
		generateFunctionDef(*it);
	}

	for(GlobalVariables::const_iterator it1 = query->getVariables().begin();
	    it1 != query->getVariables().end(); ++it1) {
		generateGlobalVar(*it1);
	}

	if(query->getQueryBody() != 0) {
		generate(query->getQueryBody()).markSubtreeResult();
	}
}

void ImpliedSchemaGenerator::generateFunctionDef(XQUserFunction *item)
{
	if(item->getFunctionBody() != 0) {
		const XQUserFunction::ArgumentSpecs *params = item->getArgumentSpecs();
		if(params) {
			varStore_.addScope(VarStore::MyScope::LOCAL_SCOPE);

			for(XQUserFunction::ArgumentSpecs::const_iterator it = params->begin();
			    it != params->end(); ++it) {
				PathResult paramRes;
				createAnyNodeResult(paramRes);
				setVariable((*it)->getURI(), (*it)->getName(), paramRes);
			}
		}

		generate(const_cast<ASTNode*>(item->getFunctionBody()));

		if(params) {
			delete varStore_.popScope();
		}
	}
}

void ImpliedSchemaGenerator::generateGlobalVar(XQGlobalVariable *item)
{
	PathResult result;

	if(item->getVariableExpr()) {
		result.join(generate(const_cast<ASTNode *>(item->getVariableExpr())));
		addSecondaryOp(result);
	} else {
		createAnyNodeResult(result);
	}

	varStore_.setGlobalVar(item->getVariableURI(), item->getVariableLocalName(), result);
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generate(ASTNode *item)
{
	PathResult result;
	switch(item->getType()) {
	case ASTNode::LITERAL: {
		result = generateLiteral((XQLiteral *)item);
		break;
	}
	case ASTNode::SEQUENCE: {
		result = generateSequence((XQSequence *)item);
		break;
	}
	case ASTNode::FUNCTION: {
		result = generateFunction((XQFunction *)item);
		break;
	}
	case ASTNode::NAVIGATION: {
		result = generateNav((XQNav *)item);
		break;
	}
	case ASTNode::VARIABLE: {
		result = generateVariable((XQVariable *)item);
		break;
	}
	case ASTNode::STEP: {
		result = generateStep((XQStep *)item);
		break;
	}
	case ASTNode::IF: {
		result = generateIf((XQIf *)item);
		break;
	}
	case ASTNode::INSTANCE_OF: {
		result = generateInstanceOf((XQInstanceOf *)item);
		break;
	}
	case ASTNode::CASTABLE_AS: {
		result = generateCastableAs((XQCastableAs *)item);
		break;
	}
	case ASTNode::CAST_AS: {
		result = generateCastAs((XQCastAs *)item);
		break;
	}
	case ASTNode::TREAT_AS: {
		result = generateTreatAs((XQTreatAs *)item);
		break;
	}
	case ASTNode::PARENTHESIZED: {
		result = generateParenthesizedExpr((XQParenthesizedExpr *)item);
		break;
	}
	case ASTNode::OPERATOR: {
		result = generateOperator((XQOperator *)item);
		break;
	}
	case ASTNode::CONTEXT_ITEM: {
		result = generateContextItem((XQContextItem *)item);
		break;
	}
	case ASTNode::RETURN: {
		result = generateReturn((XQReturn *)item);
		break;
	}
	case ASTNode::QUANTIFIED: {
		result = generateQuantified((XQQuantified *)item);
		break;
	}
	case ASTNode::TYPESWITCH: {
		result = generateTypeswitch((XQTypeswitch *)item);
		break;
	}
	case ASTNode::VALIDATE: {
		result = generateValidate((XQValidate *)item);
		break;
	}
	case ASTNode::DOM_CONSTRUCTOR: {
		result = generateDOMConstructor((XQDOMConstructor *)item);
		break;
	}
	case ASTNode::NAME_EXPRESSION: {
		result = generateNameExpression((XQNameExpression *)item);
		break;
	}
	case ASTNode::CONTENT_SEQUENCE: {
		result = generateContentSequence((XQContentSequence *)item);
		break;
	}
	case ASTNode::DIRECT_NAME: {
		result = generateDirectName((XQDirectName *)item);
		break;
	}
	case ASTNode::ORDERING_CHANGE: {
		result = generateOrderingChange((XQOrderingChange *)item);
		break;
	}
	case ASTNode::ATOMIZE: {
		result = generateAtomize((XQAtomize *)item);
		break;
	}
	case ASTNode::PROMOTE_UNTYPED: {
		result = generatePromoteUntyped((XQPromoteUntyped *)item);
		break;
	}
	case ASTNode::PROMOTE_NUMERIC: {
		result = generatePromoteNumeric((XQPromoteNumeric *)item);
		break;
	}
	case ASTNode::PROMOTE_ANY_URI: {
		result = generatePromoteAnyURI((XQPromoteAnyURI *)item);
		break;
	}
	case ASTNode::DOCUMENT_ORDER: {
		result = generateDocumentOrder((XQDocumentOrder *)item);
		break;
	}
	case ASTNode::PREDICATE: {
		result = generatePredicate((XQPredicate *)item);
		break;
	}
	case ASTNode::USER_FUNCTION: {
		result = generateUserFunction((XQUserFunction::Instance *)item);
		break;
	}
	case ASTNode::UDELETE: {
		result = generateUDelete((UDelete *)item);
		break;
	}
	case ASTNode::URENAME: {
		result = generateURename((URename *)item);
		break;
	}
	case ASTNode::UREPLACE: {
		result = generateUReplace((UReplace *)item);
		break;
	}
	case ASTNode::UREPLACE_VALUE_OF: {
		result = generateUReplaceValueOf((UReplaceValueOf *)item);
		break;
	}
	case ASTNode::UINSERT_AS_FIRST: {
		result = generateUInsertAsFirst((UInsertAsFirst *)item);
		break;
	}
	case ASTNode::UINSERT_AS_LAST: {
		result = generateUInsertAsLast((UInsertAsLast *)item);
		break;
	}
	case ASTNode::UINSERT_INTO: {
		result = generateUInsertInto((UInsertInto *)item);
		break;
	}
	case ASTNode::UINSERT_AFTER: {
		result = generateUInsertAfter((UInsertAfter *)item);
		break;
	}
	case ASTNode::UINSERT_BEFORE: {
		result = generateUInsertBefore((UInsertBefore *)item);
		break;
	}
	case ASTNode::UTRANSFORM: {
		result = generateUTransform((UTransform *)item);
		break;
	}
	case ASTNode::UAPPLY_UPDATES: {
		result = generateUApplyUpdates((UApplyUpdates *)item);
		break;
	}
	default: break;
	}
	return result;
}

static ImpliedSchemaNode::Type getTypeFromJoin(Join::Type join)
{
	ImpliedSchemaNode::Type result = (ImpliedSchemaNode::Type)-1;
	switch(join) {
	case Join::CHILD: {
		result = ImpliedSchemaNode::CHILD;
		break;
	}
	case Join::ATTRIBUTE: {
		result = ImpliedSchemaNode::ATTRIBUTE;
		break;
	}
	case Join::ATTRIBUTE_OR_CHILD: {
		result = ImpliedSchemaNode::CHILD;
		break;
	}
	case Join::DESCENDANT: {
		result = ImpliedSchemaNode::DESCENDANT;
		break;
	}
	default: {
		// Do nothing
		break;
	}
	}
	return result;
}

const ImpliedSchemaGenerator::PathResult &ImpliedSchemaGenerator::getCurrentContext() const
{
	return varStore_.getVar(XMLChDot, XMLChDot)->getValue();
}

void ImpliedSchemaGenerator::setCurrentContext(const PathResult &value)
{
	setVariable(XMLChDot, XMLChDot, value);
}

void ImpliedSchemaGenerator::setVariable(const XMLCh *uri, const XMLCh *name, const PathResult &value)
{
	varStore_.declareVar(uri, name, value);
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateStep(XQStep *item)
{
	return generateStep((Join::Type)item->getAxis(), (const DbXmlNodeTest*)item->getNodeTest(), item);
}

static inline void addPaths(ASTNode *item, const ImpliedSchemaNode::Vector &paths, XPath2MemoryManager *mm)
{
	DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
	if(ud == 0) {
		ud = new (mm) DbXmlUserData(mm);
		item->setUserData(ud);
	}
	ud->paths.insert(ud->paths.end(), paths.begin(), paths.end());
}

static inline void addPaths2(ASTNode *item, const ImpliedSchemaNode::Vector &paths, XPath2MemoryManager *mm)
{
	DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
	if(ud == 0) {
		ud = new (mm) DbXmlUserData(mm);
		item->setUserData(ud);
	}
	ud->paths2.insert(ud->paths2.end(), paths.begin(), paths.end());
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateStep(Join::Type join, const DbXmlNodeTest *step,
	ASTNode *item)
{
	ImpliedSchemaNode::Type type = getTypeFromJoin(join);

	PathResult result;

	ImpliedSchemaNode dummy(step, type, xpc_->getMemoryManager());
	const PathResult &currentContext = getCurrentContext();
	for(ImpliedSchemaNode::Vector::const_iterator it = currentContext.returnPaths.begin();
	    it != currentContext.returnPaths.end(); ++it) {

		switch(join) {
		case Join::PARENT_A:
		case Join::PARENT_C:
		case Join::PARENT: {
			generateParentStep(*it, dummy, result);
			break;
		}
		case Join::DESCENDANT_OR_SELF: {
			generateDescendantOrSelfStep(*it, dummy, result);
			break;
		}
		case Join::SELF: {
			generateSelfStep(*it, dummy, result);
			break;
		}
		case Join::ANCESTOR: {
			generateAncestorStep(*it, dummy, result);
			break;
		}
		case Join::ANCESTOR_OR_SELF: {
			generateAncestorOrSelfStep(*it, dummy, result);
			break;
		}
		case Join::FOLLOWING: {
			generateFollowingStep(*it, dummy, result);
			break;
		}
		case Join::PRECEDING: {
			generatePrecedingStep(*it, dummy, result);
			break;
		}
		case Join::PRECEDING_SIBLING:
		case Join::FOLLOWING_SIBLING: {
			generateSiblingStep(*it, dummy, result);
			break;
		}
		case Join::ATTRIBUTE_OR_CHILD:
		case Join::ATTRIBUTE:
		case Join::CHILD:
		case Join::DESCENDANT: {
			generateBuiltInStep(*it, dummy, result);
			break;
		}
		case Join::NAMESPACE:
		default: {
			// Do nothing
			break;
		}
		}
	}

	addPaths(item, result.returnPaths, xpc_->getMemoryManager());

	return result;
}

void ImpliedSchemaGenerator::generateBuiltInStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	switch(target->getType()) {
	case ImpliedSchemaNode::ATTRIBUTE: {
		// Does nothing
		break;
	}
	case ImpliedSchemaNode::CHILD:
	case ImpliedSchemaNode::DESCENDANT:
	case ImpliedSchemaNode::ROOT: {
		ImpliedSchemaNode *newNode = node.copy();
		result.join(target->appendChild(newNode));
		break;
	}
	default: break;
	}
}

void ImpliedSchemaGenerator::generateParentStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	ImpliedSchemaNode *parent = target->getParent();

	switch(target->getType()) {
	case ImpliedSchemaNode::ATTRIBUTE:
	case ImpliedSchemaNode::CHILD: {
		if(parent->isSubsetOf(&node)) {
			result.join(parent);
		} else if(node.isSubsetOf(parent)) {
			ImpliedSchemaNode *parent_parent = parent->getParent();
			if(parent_parent != 0) {
				ImpliedSchemaNode *newNode = node.copy();
				newNode->setType(parent->getType());
				result.join(parent_parent->appendChild(newNode));
			}
		}
		break;
	}
	case ImpliedSchemaNode::DESCENDANT: {
		if(parent->isSubsetOf(&node)) {
			result.join(parent);
		}

		ImpliedSchemaNode *newNode = node.copy();
		newNode->setType(ImpliedSchemaNode::DESCENDANT);
		result.join(parent->appendChild(newNode));
		break;
	}
	case ImpliedSchemaNode::ROOT: {
		// There is no parent
		break;
	}
	default: break;
	}
}

void ImpliedSchemaGenerator::generateDescendantOrSelfStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	generateSelfStep(target, node, result);
	node.setType(ImpliedSchemaNode::DESCENDANT);
	generateBuiltInStep(target, node, result);
}

void ImpliedSchemaGenerator::generateSelfStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	if(target->isSubsetOf(&node)) {
		result.join(target);
	} else if(node.isSubsetOf(target)) {
		ImpliedSchemaNode *parent = target->getParent();
		if(parent != 0) {
			ImpliedSchemaNode *newNode = node.copy();
			newNode->setType(target->getType());
			result.join(parent->appendChild(newNode));
		}
	}
}

void ImpliedSchemaGenerator::generateAncestorStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	ImpliedSchemaNode *parent = target->getParent();

	switch(target->getType()) {
	case ImpliedSchemaNode::ATTRIBUTE:
	case ImpliedSchemaNode::CHILD: {
		if(parent->isSubsetOf(&node)) {
			result.join(parent);
		} else if(node.isSubsetOf(parent)) {
			ImpliedSchemaNode *parent_parent = parent->getParent();
			if(parent_parent != 0) {
				ImpliedSchemaNode *newNode = node.copy();
				newNode->setType(parent->getType());
				result.join(parent_parent->appendChild(newNode));
			}
		}
		generateAncestorStep(parent, node, result);
		break;
	}
	case ImpliedSchemaNode::DESCENDANT: {
		if(parent->isSubsetOf(&node)) {
			result.join(parent);
		}

		ImpliedSchemaNode *newNode = node.copy();
		newNode->setType(ImpliedSchemaNode::DESCENDANT);
		result.join(parent->appendChild(newNode));

		generateAncestorStep(parent, node, result);
		break;
	}
	case ImpliedSchemaNode::ROOT: {
		// Has no ancestors
		break;
	}
	default: break;
	}
}

void ImpliedSchemaGenerator::generateAncestorOrSelfStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	generateSelfStep(target, node, result);
	generateAncestorStep(target, node, result);
}

void ImpliedSchemaGenerator::generateFollowingStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	ImpliedSchemaNode *newNode = node.copy();
	newNode->setType(ImpliedSchemaNode::DESCENDANT);
	result.join(target->getRoot()->appendChild(newNode));
}

void ImpliedSchemaGenerator::generatePrecedingStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	ImpliedSchemaNode *root = target->getRoot();

	if(root->isSubsetOf(&node)) {
		result.join(root);
	}

	ImpliedSchemaNode *newNode = node.copy();
	newNode->setType(ImpliedSchemaNode::DESCENDANT);
	result.join(root->appendChild(newNode));
}

void ImpliedSchemaGenerator::generateSiblingStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node, PathResult &result)
{
	ImpliedSchemaNode *parent = target->getParent();

	switch(target->getType()) {
	case ImpliedSchemaNode::CHILD:
	case ImpliedSchemaNode::DESCENDANT: {
		ImpliedSchemaNode *newNode = node.copy();
		newNode->setType(target->getType());
		result.join(parent->appendChild(newNode));
		break;
	}
	case ImpliedSchemaNode::ATTRIBUTE:
	case ImpliedSchemaNode::ROOT: {
		// Have no siblings
		break;
	}
	default: break;
	}
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateNav(XQNav *item)
{
	PathResult result = getCurrentContext();

	bool setContext = false;
	XQNav::Steps &args = const_cast<XQNav::Steps &>(item->getSteps());
	for(XQNav::Steps::iterator i = args.begin(); i != args.end(); ++i) {
		if(setContext) {
			varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);
			setCurrentContext(result);
		}

		result = generate(i->step);

		if(setContext) {
			delete varStore_.popScope();
		}
		setContext = true;
	}

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateFunction(XQFunction *item)
{
	VectorOfASTNodes &args = const_cast<VectorOfASTNodes &>(item->getArguments());
	const XMLCh *uri = item->getFunctionURI();
	const XMLCh *name = item->getFunctionName();

	PathResult result;

	if(uri == XQFunction::XMLChFunctionURI) {

		// fn:root()
		if(name == FunctionRoot::name) {
			const PathResult &currentContext = getCurrentContext();

			// root() gets the root of it's argument, if it has one.
			// Otherwise it gets the root of the current context
			PathResult ret;
			if(args.empty()) {
				ret = currentContext;
			}
			else {
				ret = generate(args[0]);
			}

			for(ImpliedSchemaNode::Vector::iterator it = ret.returnPaths.begin();
			    it != ret.returnPaths.end(); ++it) {
				result.join((*it)->getRoot());
			}

			addPaths(item, result.returnPaths, xpc_->getMemoryManager());
		}

		// fn:doc() and fn:collection()
		else if(name == FunctionDoc::name ||
			name == FunctionCollection::name) {
			if(!args.empty()) {
				generate(args[0]).markSubtreeValue();
			}

			DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
			if(ud != 0) {
				result.join(ud->paths);
			} else {
				DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::document_string);
				ImpliedSchemaNode *root = new (xpc_->getMemoryManager())
					ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, xpc_->getMemoryManager());
				roots_.push_back(root);
				result.join(root);
				addPaths(item, result.returnPaths, xpc_->getMemoryManager());
			}
		}

		else if(name == FunctionTrace::name) {
			// trace() returns it's first argument, and prints the string
			// value of it's second argument.

			result = generate(args[0]);
			generate(args[1]).markSubtreeValue();
		}
		else if(name == FunctionZeroOrOne::name ||
			name == FunctionOneOrMore::name ||
			name == FunctionExactlyOne::name) {
			// return their argument, or raise an exception

			result = generate(args[0]);
		}
		else if(name == FunctionInsertBefore::name) {
			// behaves a bit like union

			PathResult ret = generate(args[0]);
			result.join(ret);

			generate(args[1]).markSubtreeValue();

			PathResult ret2 = generate(args[2]);
			result.join(ret2);
		}
		else if(name == FunctionRemove::name) {
			result = generate(args[0]);
			generate(args[1]).markSubtreeValue();
		}
		else if(name == FunctionReverse::name ||
			name == FunctionUnordered::name) {
			result = generate(args[0]);
		}
		else if(name == FunctionSubsequence::name) {
			result = generate(args[0]);
			generate(args[1]).markSubtreeValue();
			if(args.size() > 2) {
				generate(args[2]).markSubtreeValue();
			}
		}
		else if(name == FunctionNamespaceURIForPrefix::name) {
			generate(args[0]).markSubtreeValue();
			addSecondaryOp(generate(args[1]));
		}
		else if(name == FunctionInScopePrefixes::name) {
			addSecondaryOp(generate(args[0]));
		}
		else if(name == FunctionDeepEqual::name) {
			generate(args[0]).markSubtreeResult();
			generate(args[1]).markSubtreeResult();
			if(args.size() > 2) {
				generate(args[2]).markSubtreeValue();
			}
		}
		else if(name == FunctionId::name ||
			name == FunctionIdref::name) {
			PathResult nodes;
			if(args.size() == 1) {
				const PathResult &currentContext = getCurrentContext();
				nodes = currentContext;
			}
			else {
				nodes = generate(args[1]);
			}

			// We don't handle these, so we mark the trees as unoptimisable
			nodes.markRoot();

			generate(args[0]).markSubtreeValue();
		}
		else if(name == FunctionLang::name) {
			XPath2MemoryManager *mm = xpc_->getMemoryManager();

			generate(args[0]).markSubtreeValue();

			PathResult nodeResult;
			if(args.size() == 1) nodeResult = getCurrentContext();
			else nodeResult = generate(args[1]);

			// We want all the xml:lang attributes from all our lookup node's ancestors

			DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::element_string);
			ImpliedSchemaNode dummy(nt, ImpliedSchemaNode::CHILD, mm);
			PathResult ancestorResult;
			ImpliedSchemaNode::Vector::const_iterator it;
			for(it = nodeResult.returnPaths.begin(); it != nodeResult.returnPaths.end(); ++it) {
				generateAncestorOrSelfStep(*it, dummy, ancestorResult);
			}

			DbXmlNodeTest *nt2 = new (mm) DbXmlNodeTest(Node::attribute_string,
				XMLUni::fgXMLURIName, mm->getPooledString("lang"), mm);
			ImpliedSchemaNode dummy2(nt2, ImpliedSchemaNode::ATTRIBUTE, mm);
			PathResult attrResult;
			for(it = ancestorResult.returnPaths.begin(); it != ancestorResult.returnPaths.end(); ++it) {
				generateBuiltInStep(*it, dummy2, attrResult);
			}

			attrResult.markSubtreeValue();
		}
		else if(name == FunctionBaseURI::name) {
			XPath2MemoryManager *mm = xpc_->getMemoryManager();

			PathResult nodeResult;
			if(args.size() == 0) nodeResult = getCurrentContext();
			else nodeResult = generate(args[0]);

			// We want all the xml:base attributes from all our lookup node's ancestors

			DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::element_string);
			ImpliedSchemaNode dummy(nt, ImpliedSchemaNode::CHILD, mm);
			PathResult ancestorResult;
			ImpliedSchemaNode::Vector::const_iterator it;
			for(it = nodeResult.returnPaths.begin(); it != nodeResult.returnPaths.end(); ++it) {
				generateAncestorOrSelfStep(*it, dummy, ancestorResult);
			}

			DbXmlNodeTest *nt2 = new (mm) DbXmlNodeTest(Node::attribute_string,
				XMLUni::fgXMLURIName, mm->getPooledString("base"), mm);
			ImpliedSchemaNode dummy2(nt2, ImpliedSchemaNode::ATTRIBUTE, mm);
			PathResult attrResult;
			for(it = ancestorResult.returnPaths.begin(); it != ancestorResult.returnPaths.end(); ++it) {
				generateBuiltInStep(*it, dummy2, attrResult);
			}

			attrResult.markSubtreeValue();
		}

		// These implicitly use the value of the current context if they have no parameters
		else if((name == FunctionStringLength::name ||
			 name == FunctionNormalizeSpace::name ||
			 name == FunctionNumber::name) &&
			args.empty()) {
			const PathResult &currentContext = getCurrentContext();
			currentContext.markSubtreeValue();
		}

		// These implicitly use the current context if they have no parameters
		else if((name == FunctionName::name ||
			 name == FunctionLocalname::name ||
			 name == FunctionNamespaceUri::name) &&
			args.empty()) {
			// Do nothing
		}

		// These use the presence of a node, not it's value (and hence sub tree)
		else if(name == FunctionBoolean::name ||
			name == FunctionExists::name) {
			generate(args[0]);
		}

		// These use the presence of a node, not it's value (and hence sub tree)
		else if(name == FunctionNot::name ||
			name == FunctionName::name ||
			name == FunctionLocalname::name ||
			name == FunctionNamespaceUri::name ||
			name == FunctionEmpty::name ||
			name == FunctionCount::name ||
			name == FunctionNilled::name) {
			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				addSecondaryOp(generate(*i));
			}
		}

		// These return the value of a node
		else if((name == FunctionString::name ||
			 name == FunctionNumber::name) &&
			args.empty()) {
			const PathResult &currentContext = getCurrentContext();
			currentContext.markSubtreeValue();
		}
		else if(name == FunctionString::name ||
			name == FunctionNumber::name ||
			name == FunctionDistinctValues::name ||
			name == FunctionData::name ||
			name == FunctionMax::name ||
			name == FunctionMin::name) {
			PathResult ret = generate(args[0]);
			ret.markSubtreeValue();

			if(args.size() > 1) {
				generate(args[1]).markSubtreeValue();
			}
		}

		// These generate substring index lookups
		else if(name == FunctionStartsWith::name) {
			generateSubstring(ImpliedSchemaNode::PREFIX, args, result, item);
		}
		else if(name == FunctionEndsWith::name) {
			generateSubstring(ImpliedSchemaNode::SUFFIX, args, result, item);
		}
		else if(name == FunctionContains::name) {
			generateSubstring(ImpliedSchemaNode::SUBSTRING, args, result, item);
		}

		else if(name == FunctionPosition::name ||
			name == FunctionLast::name) {
			// Do nothing
		}

		// fn:DbXmlDocAvailable()
		else if(name == DbXmlDocAvailable::name) {
			generate(args[0]).markSubtreeValue();
		}

		else {
			// It's in the xquery function namespace, so assume it uses values from the returned paths
			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeValue();
			}
		}

	}

	else if(uri == XQillaFunction::XMLChFunctionURI) {
		if(name == FunctionParseXML::name) {
			// Returns a document, much the same as fn:doc()
			generate(args[0]).markSubtreeValue();

			DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
			if(ud != 0) {
				result.join(ud->paths);
			} else {
				DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::document_string);
				ImpliedSchemaNode *root = new (xpc_->getMemoryManager())
					ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, xpc_->getMemoryManager());
				roots_.push_back(root);

				root->setQueryPlanRoot(((Manager&)GET_CONFIGURATION(xpc_)->getManager()).
					getDefaultQueryPlanRoot());

				result.join(root);
				addPaths(item, result.returnPaths, xpc_->getMemoryManager());
			}
		}

		else if(name == FunctionParseJSON::name) {
			// Returns a document, much the same as fn:doc()
			generate(args[0]).markSubtreeValue();

			DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
			if(ud != 0) {
				result.join(ud->paths);
			} else {
				DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::element_string);
				ImpliedSchemaNode *root = new (xpc_->getMemoryManager())
					ImpliedSchemaNode(nt, ImpliedSchemaNode::CHILD, xpc_->getMemoryManager());
				roots_.push_back(root);

				root->setQueryPlanRoot(((Manager&)GET_CONFIGURATION(xpc_)->getManager()).
					getDefaultQueryPlanRoot());

				result.join(root);
				addPaths(item, result.returnPaths, xpc_->getMemoryManager());
			}
		}

		else {
			// We don't know about this function, assume it will only need the descendants of it's node arguments
			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeResult();
			}
		}
	}

	else if(uri == DbXmlFunction::XMLChFunctionURI) {
		// dbxml:metadata()
		if(name == MetaDataFunction::name) {
			MetaDataFunction *mdFunc = (MetaDataFunction*)item;

			generate(args[0]).markSubtreeValue();

			PathResult targets;
			if(args.size() == 1) {
				const PathResult &currentContext = getCurrentContext();
				targets = currentContext;
			}
			else {
				targets = generate(args[1]);
			}

			const XMLCh *uri, *name;
			if(mdFunc->getConstantQNameArg(1, uri, name, xpc_)) {

				for(ImpliedSchemaNode::Vector::const_iterator it = targets.returnPaths.begin();
				    it != targets.returnPaths.end(); ++it) {
					DbXmlNodeTest *nt = new (xpc_->getMemoryManager())
						DbXmlNodeTest(Node::document_string, uri, name, xpc_->getMemoryManager());

					ImpliedSchemaNode *newNode = new (xpc_->getMemoryManager())
						ImpliedSchemaNode(nt, ImpliedSchemaNode::METADATA,
							xpc_->getMemoryManager());
					result.join((*it)->appendChild(newNode));
				}

				addPaths(item, result.returnPaths, xpc_->getMemoryManager());
			}
		}

		else if(name == LookupIndexFunction::name) {
			LookupIndexFunction *fun = (LookupIndexFunction*)item;

			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeValue();
			}

			generateLookup(fun, fun, ImpliedSchemaNode::CHILD, fun->getChildURIName(),
				fun->getParentURIName(), result);
		}

		else if(name == LookupAttributeIndexFunction::name) {
			LookupAttributeIndexFunction *fun = (LookupAttributeIndexFunction*)item;

			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeValue();
			}

			generateLookup(fun, fun, ImpliedSchemaNode::ATTRIBUTE, fun->getChildURIName(),
				fun->getParentURIName(), result);
		}

		else if(name == LookupMetaDataIndexFunction::name) {
			LookupMetaDataIndexFunction *fun = (LookupMetaDataIndexFunction*)item;

			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeValue();
			}

			generateLookup(fun, fun, ImpliedSchemaNode::METADATA, fun->getChildURIName(),
				fun->getParentURIName(), result);
		}

		else if(name == DbXmlContainsFunction::name) {
			generateSubstring(ImpliedSchemaNode::SUBSTRING_CD, args, result, item);
		}

		else if(name == HandleToNodeFunction::name) {
			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeValue();
			}

			createAnyNodeResult(result);
		}

		else {
			// We don't know about this function, assume it will only need the descendants of it's node arguments
			for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
				generate(*i).markSubtreeResult();
			}
		}
	}

	else {
		// We don't know about this function, assume it will only need the descendants of it's node arguments
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			generate(*i).markSubtreeResult();
		}
	}

	return result;
}

class ArgHolder {
public:
	ArgHolder(const XMLCh *u, const XMLCh *n, const ImpliedSchemaGenerator::PathResult &v)
		: uri(u), name(n), value(v) {}
	
	const XMLCh *uri, *name;
	ImpliedSchemaGenerator::PathResult value;
};

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUserFunction(XQUserFunction::Instance *item)
{
	PathResult result;

	VectorOfASTNodes &args = const_cast<VectorOfASTNodes &>(item->getArguments());
	const XQUserFunction *functionDef = item->getFunctionDefinition();

	// Evaluate the arguments in the current scope, declare them in the function's scope
	vector<ArgHolder> evaluatedArgs;
	if(functionDef->getArgumentSpecs()) {
		XQUserFunction::ArgumentSpecs::const_iterator binding = functionDef->getArgumentSpecs()->begin();
		for(VectorOfASTNodes::iterator arg = args.begin(); arg != args.end() && binding != functionDef->getArgumentSpecs()->end(); ++arg, ++binding) {

			PathResult r = generate(*arg);
			addSecondaryOp(r);

			if((*binding)->isUsed()) {
				evaluatedArgs.push_back(ArgHolder((*binding)->getURI(), (*binding)->getName(), r));
			}
		}
	}

	if(functionDef->getFunctionBody() != 0) {
		// Check to see if this user function is recursing into a user function
		// that has already been called.
		pair<set<const ASTNode*>::iterator, bool> ins = userFunctionStack_.insert(functionDef->getFunctionBody());
		if(ins.second) {
			varStore_.addScope(VarStore::MyScope::LOCAL_SCOPE);

			for(vector<ArgHolder>::iterator it = evaluatedArgs.begin(); it != evaluatedArgs.end(); ++it) {
				setVariable(it->uri, it->name, it->value);
			}

			PathResult ret = generate(const_cast<ASTNode *>(functionDef->getFunctionBody()));
			result.join(ret);

			delete varStore_.popScope();

			userFunctionStack_.erase(ins.first);
			return result;
		}
	}

	// All we can really do if it is recursive is to mark the entire tree as needed
	// TBD see if we can handle recursive user functions better - jpcs
	for(vector<ArgHolder>::iterator it = evaluatedArgs.begin(); it != evaluatedArgs.end(); ++it) {
		it->value.markRoot();
	}

	return result;
}

void ImpliedSchemaGenerator::generateLookup(ASTNode *item, QueryPlanRoot *qpr, ImpliedSchemaNode::Type type,
	const char *child, const char *parent, PathResult &result)
{
	DbXmlUserData *ud = (DbXmlUserData*)item->getUserData();
	if(ud != 0) {
		result.join(ud->paths);
		return;
	}

	XPath2MemoryManager *mm = xpc_->getMemoryManager();

	DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::document_string);
        ImpliedSchemaNode *root = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, mm);
	roots_.push_back(root);
        root->setQueryPlanRoot(qpr);

	const ContainerBase *container = qpr->getContainerBase();

	if(type == ImpliedSchemaNode::METADATA || container == 0 || !container->nodesIndexed()) {
		result.join(root);
	}

	if(type != ImpliedSchemaNode::METADATA && (container == 0 || container->nodesIndexed())) {
		ImpliedSchemaNode *pisn = 0;
		if(parent == 0 || *parent == 0) {
			if(type == ImpliedSchemaNode::ATTRIBUTE) {
				DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::element_string);
				pisn = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::DESCENDANT, mm);
				pisn = root->appendChild(pisn);
			} else {
				pisn = root;
				type = ImpliedSchemaNode::DESCENDANT;
			}
		} else {
			Name pname(parent);
			const char *puri = pname.hasURI() ? pname.getURI() : 0;
			DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(Node::element_string,
				mm->getPooledString(puri), mm->getPooledString(pname.getName()), xpc_->getMemoryManager());
			pisn = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::DESCENDANT, mm);
			pisn = root->appendChild(pisn);
		}

		const XMLCh *node_type = Node::element_string;
		if(type == ImpliedSchemaNode::ATTRIBUTE) {
			node_type = Node::attribute_string;
		}

		ImpliedSchemaNode *cisn = 0;
		if(child == 0 || *child == 0) {
			DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(node_type);
			cisn = new (mm) ImpliedSchemaNode(nt, type, mm);
		} else {
			Name cname(child);
			const char *curi = cname.hasURI() ? cname.getURI() : 0;
			DbXmlNodeTest *nt = new (xpc_->getMemoryManager()) DbXmlNodeTest(node_type,
				mm->getPooledString(curi), mm->getPooledString(cname.getName()), xpc_->getMemoryManager());
			cisn = new (mm) ImpliedSchemaNode(nt, type, mm);
		}
		result.join(pisn->appendChild(cisn));
	}

	addPaths(item, result.returnPaths, xpc_->getMemoryManager());
}

static StaticType generalCompTypeConversion(const StaticType &arg0_type, const StaticType &arg1_type)
{
	StaticType result = arg0_type;
	result.flags &= ~(StaticType::UNTYPED_ATOMIC_TYPE | StaticType::NODE_TYPE);
	if(arg0_type.containsType(StaticType::UNTYPED_ATOMIC_TYPE | StaticType::NODE_TYPE)) {
		if(arg1_type.containsType(StaticType::NUMERIC_TYPE)) {
			result.flags |= StaticType::DOUBLE_TYPE;
		}
		if(arg1_type.containsType(StaticType::UNTYPED_ATOMIC_TYPE | StaticType::NODE_TYPE)) {
			result.flags |= StaticType::STRING_TYPE;
		}
		if(arg1_type.containsType(~(StaticType::NUMERIC_TYPE | StaticType::UNTYPED_ATOMIC_TYPE))) {
			result.flags |= arg1_type.flags & ~(StaticType::NUMERIC_TYPE | StaticType::UNTYPED_ATOMIC_TYPE);
		}
	}
	return result;
}

static StaticType valueCompTypeConversion(const StaticType &arg0_type)
{
	StaticType result = arg0_type;
	result.flags &= ~(StaticType::UNTYPED_ATOMIC_TYPE | StaticType::NODE_TYPE);
	if(arg0_type.containsType(StaticType::UNTYPED_ATOMIC_TYPE | StaticType::NODE_TYPE)) {
		result.flags |= StaticType::STRING_TYPE;
	}
	return result;
}

static XmlValue::Type staticTypeToValueType(unsigned int sType)
{
	switch(sType) {
	case StaticType::NODE_TYPE: return XmlValue::NODE;
	case StaticType::ANY_SIMPLE_TYPE: return XmlValue::ANY_SIMPLE_TYPE;
	case StaticType::ANY_URI_TYPE: return XmlValue::ANY_URI;
	case StaticType::BASE_64_BINARY_TYPE: return XmlValue::BASE_64_BINARY;
	case StaticType::BOOLEAN_TYPE: return XmlValue::BOOLEAN;
	case StaticType::DATE_TYPE: return XmlValue::DATE;
	case StaticType::DATE_TIME_TYPE: return XmlValue::DATE_TIME;
	case StaticType::DAY_TIME_DURATION_TYPE: return XmlValue::DAY_TIME_DURATION;
	case StaticType::DECIMAL_TYPE: return XmlValue::DECIMAL;
	case StaticType::DOUBLE_TYPE: return XmlValue::DOUBLE;
	case StaticType::DURATION_TYPE: return XmlValue::DURATION;
	case StaticType::FLOAT_TYPE: return XmlValue::FLOAT;
	case StaticType::G_DAY_TYPE: return XmlValue::G_DAY;
	case StaticType::G_MONTH_TYPE: return XmlValue::G_MONTH;
	case StaticType::G_MONTH_DAY_TYPE: return XmlValue::G_MONTH_DAY;
	case StaticType::G_YEAR_TYPE: return XmlValue::G_YEAR;
	case StaticType::G_YEAR_MONTH_TYPE: return XmlValue::G_YEAR_MONTH;
	case StaticType::HEX_BINARY_TYPE: return XmlValue::HEX_BINARY;
	case StaticType::NOTATION_TYPE: return XmlValue::NOTATION;
	case StaticType::QNAME_TYPE: return XmlValue::QNAME;
	case StaticType::STRING_TYPE: return XmlValue::STRING;
	case StaticType::TIME_TYPE: return XmlValue::TIME;
	case StaticType::UNTYPED_ATOMIC_TYPE: return XmlValue::UNTYPED_ATOMIC;
	case StaticType::YEAR_MONTH_DURATION_TYPE: return XmlValue::YEAR_MONTH_DURATION;
	default: break;
	}
	return XmlValue::NONE;
}

static Syntax::Type getComparisonType(const StaticType &arg0_type, const StaticType &arg1_type)
{
	XmlValue::Type result = staticTypeToValueType(arg0_type.flags);
	if(result == XmlValue::NONE) 
		result = staticTypeToValueType(arg1_type.flags);
	return AtomicTypeValue::convertToSyntaxType(result);
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateOperator(XQOperator *item)
{
	PathResult result;

	VectorOfASTNodes &args = const_cast<VectorOfASTNodes &>(item->getArguments());
	const XMLCh *name = item->getOperatorName();

	if(name == Union::name) {
		PathResult ret = generate(args[0]);
		result.join(ret);

		ret = generate(args[1]);
		result.join(ret);
	}

	else if(name == Intersect::name) {
		PathResult ret = generate(args[0]);
		result.join(ret);

		ret = generate(args[1]);
		result.join(ret);
	}

	else if(name == Except::name) {
		result = generate(args[0]);
		addSecondaryOp(generate(args[1]));
	}

	else if(name == Equals::name) {
		generateComparison(ImpliedSchemaNode::EQUALS, /*generateComp*/false, args, result, item);
	}

	else if(name == NotEquals::name) {
		// We generate NotEquals as Equals, and deal with the negation later
		generateComparison(ImpliedSchemaNode::EQUALS, /*generateComp*/false, args, result, item);
	}

	else if(name == LessThan::name) {
		generateComparison(ImpliedSchemaNode::LTX, /*generateComp*/false, args, result, item);
	}

	else if(name == LessThanEqual::name) {
		generateComparison(ImpliedSchemaNode::LTE, /*generateComp*/false, args, result, item);
	}

	else if(name == GreaterThan::name) {
		generateComparison(ImpliedSchemaNode::GTX, /*generateComp*/false, args, result, item);
	}

	else if(name == GreaterThanEqual::name) {
		generateComparison(ImpliedSchemaNode::GTE, /*generateComp*/false, args, result, item);
	}

	else if(name == GeneralComp::name) {
		switch(((const GeneralComp *)item)->getOperation()) {
		case GeneralComp::EQUAL:
			generateComparison(ImpliedSchemaNode::EQUALS, /*generateComp*/true, args, result, item);
			break;
		case GeneralComp::NOT_EQUAL:
			// We generate NotEquals as Equals, and deal with the negation later
			generateComparison(ImpliedSchemaNode::EQUALS, /*generateComp*/true, args, result, item);
			break;
		case GeneralComp::LESS_THAN:
			generateComparison(ImpliedSchemaNode::LTX, /*generateComp*/true, args, result, item);
			break;
		case GeneralComp::LESS_THAN_EQUAL:
			generateComparison(ImpliedSchemaNode::LTE, /*generateComp*/true, args, result, item);
			break;
		case GeneralComp::GREATER_THAN:
			generateComparison(ImpliedSchemaNode::GTX, /*generateComp*/true, args, result, item);
			break;
		case GeneralComp::GREATER_THAN_EQUAL:
			generateComparison(ImpliedSchemaNode::GTE, /*generateComp*/true, args, result, item);
			break;
		}
	}

	// These operators use the value of their arguments
	else if(name == Divide::name ||
		name == IntegerDivide::name ||
		name == Minus::name ||
		name == Mod::name ||
		name == Multiply::name ||
		name == Plus::name ||
		name == Range::name ||
		name == UnaryMinus::name) {
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			generate(*i).markSubtreeValue();
		}
	}

	else if(name == And::name) {
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			PathResult ret = generate(*i);
		}
	}

	else if(name == Or::name) {
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			PathResult ret = generate(*i);
		}
	}

	// These operators use the presence of the node arguments, not their value
	else if(name == NodeComparison::name ||
		name == OrderComparison::name) {
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			addSecondaryOp(generate(*i));
		}
	}

	else {
		// Better to be safer than sorry
		for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
			PathResult ret = generate(*i);
			ret.markRoot();
			addSecondaryOp(ret);
		}
	}

	return result;
}

static ImpliedSchemaNode::Type oppositeOperation(ImpliedSchemaNode::Type type)
{
	switch(type) {
	case ImpliedSchemaNode::LTX: {
		return ImpliedSchemaNode::GTX;
	}
	case ImpliedSchemaNode::LTE: {
		return ImpliedSchemaNode::GTE;
	}
	case ImpliedSchemaNode::GTX: {
		return ImpliedSchemaNode::LTX;
	}
	case ImpliedSchemaNode::GTE: {
		return ImpliedSchemaNode::LTE;
	}
	case ImpliedSchemaNode::EQUALS: {
		return ImpliedSchemaNode::EQUALS;
	}
	default: break;
	}
	return (ImpliedSchemaNode::Type)-1;
}

void ImpliedSchemaGenerator::generateComparison(ImpliedSchemaNode::Type type, bool generalComp,
	VectorOfASTNodes &args, PathResult &result, ASTNode *item)
{
	ASTNode *argA = args[0];
	ASTNode *argB = args[1];

	PathResult retA = generate(argA);
	PathResult retB = generate(argB);

	StaticType arg0_type = argA->getStaticAnalysis().getStaticType();
	StaticType arg1_type = argB->getStaticAnalysis().getStaticType();

	Syntax::Type syntaxType;
	if(generalComp) {
		syntaxType = getComparisonType(generalCompTypeConversion(arg0_type, arg1_type),
			generalCompTypeConversion(arg1_type, arg0_type));
	}
	else {
		syntaxType = getComparisonType(valueCompTypeConversion(arg0_type),
			valueCompTypeConversion(arg1_type));
	}

	XPath2MemoryManager *mm = xpc_->getMemoryManager();
	ImpliedSchemaNode::Vector::iterator it;
	for(it = retA.returnPaths.begin(); it != retA.returnPaths.end(); ++it) {
		if(type != (ImpliedSchemaNode::Type)-1) {
			ImpliedSchemaNode *newChild = (*it)->appendChild(new (mm)
				ImpliedSchemaNode(type, generalComp, syntaxType, argB, mm));
			result.join(newChild);
		} else (*it)->markSubtreeValue();
	}

	addPaths(item, result.returnPaths, xpc_->getMemoryManager());

	// We don't want to return any ImpliedSchemaNodes with a type
	// other than child, attribute, or descendant.
	result.returnPaths.clear();

	for(it = retB.returnPaths.begin(); it != retB.returnPaths.end(); ++it) {
		if(type != (ImpliedSchemaNode::Type)-1) {
			ImpliedSchemaNode *newChild = (*it)->appendChild(new (mm)
				ImpliedSchemaNode(oppositeOperation(type), generalComp,
					syntaxType, argA, mm));
			result.join(newChild);
		} else (*it)->markSubtreeValue();
	}

	addPaths2(item, result.returnPaths, xpc_->getMemoryManager());

	// We don't want to return any ImpliedSchemaNodes with a type
	// other than child, attribute, or descendant.
	result.returnPaths.clear();
}

void ImpliedSchemaGenerator::addSecondaryOp(const PathResult &result)
{
}

void ImpliedSchemaGenerator::generateSubstring(ImpliedSchemaNode::Type type, VectorOfASTNodes &args, PathResult &result,
	ASTNode *item)
{
	if(args.size() > 2) {
		// TBD case/diacritic insensitive collation? - jpcs

		// Mark the collation argument
		generate(args[2]).markSubtreeValue();
	}

	PathResult ret = generate(args[0]);
	generate(args[1]).markSubtreeValue();

	XPath2MemoryManager *mm = xpc_->getMemoryManager();
	for(ImpliedSchemaNode::Vector::iterator it = ret.returnPaths.begin(); it != ret.returnPaths.end(); ++it) {
		ImpliedSchemaNode *newChild = (*it)->appendChild(new (mm)
			ImpliedSchemaNode(type, /*generalComp*/false, Syntax::STRING, args[1], mm));
		result.join(newChild);
	}
	addPaths(item, result.returnPaths, xpc_->getMemoryManager());

	// We don't want to return any ImpliedSchemaNodes with a type
	// other than child, attribute, or descendant.
	result.returnPaths.clear();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateParenthesizedExpr(XQParenthesizedExpr *item)
{
	VectorOfASTNodes &args = const_cast<VectorOfASTNodes &>(item->getChildren());
	PathResult result;
	for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
		PathResult ret = generate(*i);
		result.join(ret);
	}

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateVariable(XQVariable *item)
{
	VarStoreRef *ref = varStore_.getVar(item->getURI(), item->getName());

	PathResult result;
	if(ref != 0) {
		result = ref->getValue();
	} else {
		// Undeclared global variable - we'll declare it here
		PathResult vv;
		createAnyNodeResult(vv);
		varStore_.setGlobalVar(item->getURI(), item->getName(), vv);

		result = vv;
	}

	addPaths(item, result.returnPaths, xpc_->getMemoryManager());

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateIf(XQIf *item)
{
	PathResult result;

	PathResult test = generate(const_cast<ASTNode *>(item->getTest()));

	PathResult ret = generate(const_cast<ASTNode *>(item->getWhenTrue()));
	result.join(ret);

	ret = generate(const_cast<ASTNode *>(item->getWhenFalse()));
	result.join(ret);

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateInstanceOf(XQInstanceOf *item)
{
	PathResult result;
	generate(const_cast<ASTNode *>(item->getExpression())).markSubtreeValue();
	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateCastableAs(XQCastableAs *item)
{
	PathResult result;
	generate(const_cast<ASTNode *>(item->getExpression())).markSubtreeValue();
	return result;
}

void ImpliedSchemaGenerator::generateTupleNode(TupleNode *item)
{
	switch(item->getType()) {
	case TupleNode::CONTEXT_TUPLE: {
		break;
	}
	case TupleNode::FOR: {
		generateForTuple((ForTuple*)item);
		break;
	}
	case TupleNode::LET: {
		generateLetTuple((LetTuple*)item);
		break;
	}
	case TupleNode::WHERE: {
		generateWhereTuple((WhereTuple*)item);
		break;
	}
	case TupleNode::ORDER_BY: {
		generateOrderByTuple((OrderByTuple*)item);
		break;
	}
	default: break;
	}
}

void ImpliedSchemaGenerator::generateForTuple(ForTuple *item)
{
	generateTupleNode(const_cast<TupleNode*>(item->getParent()));

	PathResult r = generate(item->getExpression());

	if(item->getVarName()) {
		setVariable(item->getVarURI(), item->getVarName(), r);
	}
	if(item->getPosName()) {
		setVariable(item->getPosURI(), item->getPosName(), PathResult());
	}
}

void ImpliedSchemaGenerator::generateLetTuple(LetTuple *item)
{
	generateTupleNode(const_cast<TupleNode*>(item->getParent()));

	PathResult r = generate(item->getExpression());

	if(item->getVarName()) {
		setVariable(item->getVarURI(), item->getVarName(), r);
	}
}

void ImpliedSchemaGenerator::generateWhereTuple(WhereTuple *item)
{
	generateTupleNode(const_cast<TupleNode*>(item->getParent()));

	generate(item->getExpression());
}

void ImpliedSchemaGenerator::generateOrderByTuple(OrderByTuple *item)
{
	generateTupleNode(const_cast<TupleNode*>(item->getParent()));

	generate(item->getExpression()).markSubtreeValue();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateReturn(XQReturn *item)
{
	varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);

	generateTupleNode(const_cast<TupleNode*>(item->getParent()));
	PathResult result = generate(item->getExpression());
	
	delete varStore_.popScope();

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateQuantified(XQQuantified *item)
{
	varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);

	generateTupleNode(const_cast<TupleNode*>(item->getParent()));
	generate(item->getExpression());
	
	delete varStore_.popScope();

	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateTypeswitch(XQTypeswitch *item)
{
	PathResult result;

	PathResult expr = generate(const_cast<ASTNode *>(item->getExpression()));

	XQTypeswitch::Cases *clauses = const_cast<XQTypeswitch::Cases *>(item->getCases());
	for(XQTypeswitch::Cases::iterator i = clauses->begin(); i != clauses->end(); ++i) {

		if((*i)->isVariableUsed()) {
			varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);
			setVariable((*i)->getURI(), (*i)->getName(), expr);
		}

		PathResult ret = generate((*i)->getExpression());
		result.join(ret);

		if((*i)->isVariableUsed()) {
			delete varStore_.popScope();
		}
	}

	XQTypeswitch::Case *def = const_cast<XQTypeswitch::Case *>(item->getDefaultCase());
	if(def->isVariableUsed()) {
		varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);
		setVariable(def->getURI(), def->getName(), expr);
	}

	PathResult ret = generate(def->getExpression());
	result.join(ret);

	if(def->isVariableUsed()) {
		delete varStore_.popScope();
	}

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateDOMConstructor(XQDOMConstructor *item)
{
	PathResult result;

	if(item->getName()) {
		generate(const_cast<ASTNode *>(item->getName())).markSubtreeValue();
	}

	VectorOfASTNodes *attrs = const_cast<VectorOfASTNodes *>(item->getAttributes());
	if(attrs) {
		for(VectorOfASTNodes::iterator i = attrs->begin(); i != attrs->end(); ++i) {
			generate(*i).markSubtreeResult();
		}
	}

	VectorOfASTNodes *children = const_cast<VectorOfASTNodes *>(item->getChildren());
	if(children) {
		for(VectorOfASTNodes::iterator j = children->begin(); j != children->end(); ++j) {
			generate(*j).markSubtreeResult();
		}
	}

	if(item->getValue()) {
		generate(const_cast<ASTNode *>(item->getValue())).markSubtreeValue();
	}

	XPath2MemoryManager *mm = xpc_->getMemoryManager();

	DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::document_string);
        ImpliedSchemaNode *root = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, mm);
	roots_.push_back(root);

	root->setQueryPlanRoot(((Manager&)GET_CONFIGURATION(xpc_)->getManager()).
		getDefaultQueryPlanRoot());

	// TBD Could we find out the name of the node? - jpcs
	if(item->getNodeType() == Node::document_string) {
		result.join(root);
	} else if(item->getNodeType() == Node::element_string) {
		result.join(root->appendChild(new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(Node::element_string), ImpliedSchemaNode::DESCENDANT, mm)));
	} else if(item->getNodeType() == Node::attribute_string) {
		ImpliedSchemaNode *pisn = new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(Node::element_string), ImpliedSchemaNode::DESCENDANT, mm);
		pisn = root->appendChild(pisn);
		result.join(pisn->appendChild(new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(Node::attribute_string), ImpliedSchemaNode::ATTRIBUTE, mm)));
	} else {
		ImpliedSchemaNode *pisn = new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(0), ImpliedSchemaNode::DESCENDANT, mm);
		result.join(root->appendChild(pisn));
	}

	addPaths(item, result.returnPaths, mm);

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateNameExpression(XQNameExpression *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::copyNodes(const PathResult &r)
{
	XPath2MemoryManager *mm = xpc_->getMemoryManager();

	// Emulate copying the nodes
	PathResult copyResult;

	DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::document_string);
	ImpliedSchemaNode *root = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, mm);
	roots_.push_back(root);

	root->setQueryPlanRoot(((Manager&)GET_CONFIGURATION(xpc_)->getManager()).
		getDefaultQueryPlanRoot());

	ImpliedSchemaNode::Vector::const_iterator j = r.returnPaths.begin();
	for(; j != r.returnPaths.end(); ++j) {
		switch((*j)->getType()) {
		case ImpliedSchemaNode::ATTRIBUTE: {
			ImpliedSchemaNode *pisn = new (mm) ImpliedSchemaNode(new (mm) DbXmlNodeTest(Node::element_string),
				ImpliedSchemaNode::DESCENDANT, mm);
			pisn = root->appendChild(pisn);
			copyResult.join(pisn->appendChild(new (mm) ImpliedSchemaNode((*j)->getNodeTest(),
								  ImpliedSchemaNode::ATTRIBUTE, mm)));
			break;
		}
		case ImpliedSchemaNode::CHILD:
		case ImpliedSchemaNode::DESCENDANT: {
			copyResult.join(root->appendChild(new (mm) ImpliedSchemaNode((*j)->getNodeTest(),
								  ImpliedSchemaNode::DESCENDANT, mm)));
			break;
		}
		case ImpliedSchemaNode::ROOT: {
			copyResult.join(root);
			break;
		}
		default: break;
		}
	}

	return copyResult;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateContentSequence(XQContentSequence *item)
{
	PathResult r = generate(const_cast<ASTNode *>(item->getExpression()));
	r.markSubtreeResult();
	return copyNodes(r);
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateDirectName(XQDirectName *item)
{
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateContextItem(XQContextItem *item)
{
	const PathResult &currentContext = getCurrentContext();
	PathResult result = currentContext;
	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateLiteral(XQLiteral *item)
{
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateSequence(XQSequence *item)
{
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateCastAs(XQCastAs *item)
{
	XPath2MemoryManager *mm = xpc_->getMemoryManager();

	PathResult result;

	PathResult argResult = generate(const_cast<ASTNode*>(item->getExpression()));

	for(ImpliedSchemaNode::Vector::iterator it = argResult.returnPaths.begin();
	    it != argResult.returnPaths.end(); ++it) {
		ImpliedSchemaNode *newNode = new (mm) ImpliedSchemaNode(item, mm);
		result.join((*it)->appendChild(newNode));
	}

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateTreatAs(XQTreatAs *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateOrderingChange(XQOrderingChange *item)
{
	return generate(item->getExpr());
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateAtomize(XQAtomize *item)
{
	PathResult ret = generate(const_cast<ASTNode *>(item->getExpression()));
	ret.markSubtreeValue();
	return ret;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generatePromoteUntyped(XQPromoteUntyped *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generatePromoteNumeric(XQPromoteNumeric *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generatePromoteAnyURI(XQPromoteAnyURI *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateDocumentOrder(XQDocumentOrder *item)
{
	PathResult result = generate(const_cast<ASTNode *>(item->getExpression()));
	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateValidate(XQValidate *item)
{
	PathResult r = generate(const_cast<ASTNode *>(item->getExpression()));
	r.markSubtreeResult();
	return copyNodes(r);
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generatePredicate(XQPredicate *item)
{
	PathResult result = generate(const_cast<ASTNode *>(item->getExpression()));

	varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);
	setCurrentContext(result);

	PathResult predResult = generate(const_cast<ASTNode *>(item->getPredicate()));

	delete varStore_.popScope();
	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUDelete(UDelete *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getExpression())).markRoot();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateURename(URename *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getName())).markSubtreeValue();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUReplace(UReplace *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getExpression())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUReplaceValueOf(UReplaceValueOf *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getExpression())).markSubtreeValue();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUInsertAsFirst(UInsertAsFirst *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getSource())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUInsertAsLast(UInsertAsLast *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getSource())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUInsertInto(UInsertInto *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getSource())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUInsertAfter(UInsertAfter *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getSource())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUInsertBefore(UInsertBefore *item)
{
	// Any tree that is updated can't be document projected
	generate(const_cast<ASTNode *>(item->getTarget())).markRoot();
	generate(const_cast<ASTNode *>(item->getSource())).markSubtreeResult();
	return PathResult();
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUTransform(UTransform *item)
{
	varStore_.addScope(VarStore::MyScope::LOGICAL_BLOCK_SCOPE);

	VectorOfCopyBinding *bindings = const_cast<VectorOfCopyBinding*>(item->getBindings());
	for(VectorOfCopyBinding::iterator i = bindings->begin(); i != bindings->end(); ++i) {
		PathResult r = generate((*i)->expr_);
		setVariable((*i)->uri_, (*i)->name_, r);
	}

	generate(const_cast<ASTNode *>(item->getModifyExpr()));
	PathResult result = generate(const_cast<ASTNode *>(item->getReturnExpr()));

	delete varStore_.popScope();

	return result;
}

ImpliedSchemaGenerator::PathResult ImpliedSchemaGenerator::generateUApplyUpdates(UApplyUpdates *item)
{
	return generate(const_cast<ASTNode *>(item->getExpression()));
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void ImpliedSchemaGenerator::PathResult::join(const ImpliedSchemaNode::MVector &o)
{
	returnPaths.insert(returnPaths.begin(), o.begin(), o.end());
}

void ImpliedSchemaGenerator::PathResult::join(const PathResult &o)
{
	returnPaths.insert(returnPaths.begin(), o.returnPaths.begin(), o.returnPaths.end());
}

void ImpliedSchemaGenerator::PathResult::join(ImpliedSchemaNode *o)
{
	returnPaths.push_back(o);
}

void ImpliedSchemaGenerator::PathResult::markSubtreeValue() const
{
	// Mark the paths as needing the subtree
	for(ImpliedSchemaNode::Vector::const_iterator it = returnPaths.begin(); it != returnPaths.end(); ++it) {
		(*it)->markSubtreeValue();
	}
}

void ImpliedSchemaGenerator::PathResult::markSubtreeResult() const
{
	// Mark the paths as needing the subtree
	for(ImpliedSchemaNode::Vector::const_iterator it = returnPaths.begin(); it != returnPaths.end(); ++it) {
		(*it)->markSubtreeResult();
	}
}

void ImpliedSchemaGenerator::PathResult::markRoot() const
{
	// Mark the paths as needing the entire tree
	for(ImpliedSchemaNode::Vector::const_iterator it = returnPaths.begin(); it != returnPaths.end(); ++it) {
		(*it)->getRoot()->markSubtreeResult();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void ProjectionSchemaAdjuster::addSchemaForIndex(PresenceQP *item)
{
	XPath2MemoryManager *mm = xpc_->getMemoryManager();
	DbXmlConfiguration *conf = GET_CONFIGURATION(xpc_);
	ContainerBase *container = item->getContainerBase();

	// We need to add a document projection schema for and DLS+ index lookups,
	// to make sure that the node it points to is still in the projected tree
	if(container != 0 && container->getContainer() != 0 &&
		container->getContainer()->isWholedocContainer() && container->nodesIndexed() &&
		!item->isDocumentIndex() && item->getNodeType() != ImpliedSchemaNode::METADATA) {

		DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::document_string);
		ImpliedSchemaNode *root = new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::ROOT, mm);
		roots_.push_back(root);

		Name cname(item->getChildName());
		const char *curi = cname.hasURI() ? cname.getURI() : 0;

		switch(item->getNodeType()) {
		case ImpliedSchemaNode::ATTRIBUTE: {
			ImpliedSchemaNode::Type type = ImpliedSchemaNode::DESCENDANT_ATTR;
			ImpliedSchemaNode *pisn = root;

			if(item->isParentSet()) {
				Name pname(item->getParentName());
				const char *puri = pname.hasURI() ? pname.getURI() : 0;
				DbXmlNodeTest *nt2 = new (mm) DbXmlNodeTest(Node::element_string,
					mm->getPooledString(puri), mm->getPooledString(pname.getName()), mm);
				pisn = new (mm) ImpliedSchemaNode(nt2, ImpliedSchemaNode::DESCENDANT, mm);
				pisn = root->appendChild(pisn);
				type = ImpliedSchemaNode::ATTRIBUTE;
			}

			DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::attribute_string,
				mm->getPooledString(curi), mm->getPooledString(cname.getName()), mm);
			pisn->appendChild(new (mm) ImpliedSchemaNode(nt, type, mm));
			break;
		}
		case ImpliedSchemaNode::CHILD:
		case ImpliedSchemaNode::DESCENDANT: {
			DbXmlNodeTest *nt = new (mm) DbXmlNodeTest(Node::element_string,
				mm->getPooledString(curi), mm->getPooledString(cname.getName()), mm);
			root->appendChild(new (mm) ImpliedSchemaNode(nt, ImpliedSchemaNode::DESCENDANT, mm));
			break;
		}
		default: break;
		}

		conf->addImpliedSchemaNode(container->getContainerID(), root);
	}
}

QueryPlan *ProjectionSchemaAdjuster::optimizePresence(PresenceQP *item)
{
	addSchemaForIndex(item);

	return item;
}

QueryPlan *ProjectionSchemaAdjuster::optimizeValue(ValueQP *item)
{
	addSchemaForIndex(item);

	QPValue &value = const_cast<QPValue&>(item->getValue());
	if(value.getASTNode() != 0) {
		value.setASTNode(optimize(const_cast<ASTNode*>(value.getASTNode())));
	}
	return item;
}

QueryPlan *ProjectionSchemaAdjuster::optimizeRange(RangeQP *item)
{
	addSchemaForIndex(item);

	QPValue &value = const_cast<QPValue&>(item->getValue());
	if(value.getASTNode() != 0) {
		value.setASTNode(optimize(const_cast<ASTNode*>(value.getASTNode())));
	}
	QPValue &value2 = const_cast<QPValue&>(item->getValue2());
	if(value2.getASTNode() != 0) {
		value2.setASTNode(optimize(const_cast<ASTNode*>(value2.getASTNode())));
	}
	return item;
}

