//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: ImpliedSchemaGenerator.hpp,v 1.13 2008/01/08 21:54:24 bostic Exp $
//

#ifndef __IMPLIEDSCHEMAGENERATOR_HPP
#define	__IMPLIEDSCHEMAGENERATOR_HPP

#include <dbxml/XmlQueryContext.hpp>
#include "NodeVisitingOptimizer.hpp"
#include "../query/ImpliedSchemaNode.hpp"
#include "../dataItem/Join.hpp"

#include <xqilla/context/impl/VariableStoreTemplate.hpp>
#include <xqilla/framework/XPath2MemoryManagerImpl.hpp>

namespace DbXml
{

class DbXmlDocAvailable;
class DbXmlNodeTest;

/**
 * Generates ImpliedSchemaNode trees of the paths
 * in the documents that will be navigated. Decorates
 * the AST with the ImpliedSchemaNode objects that apply
 * at each stage.
 */
class ImpliedSchemaGenerator : public Optimizer
{
public:
	ImpliedSchemaGenerator(DynamicContext *xpc, Optimizer *parent = 0);
	virtual ~ImpliedSchemaGenerator() { varStore_.clear(); }

	const ImpliedSchemaNode::Vector & getRoots() const { return roots_; }

	class PathResult {
	public:
		void join(const ImpliedSchemaNode::MVector &o);
		void join(const PathResult &o);
		void join(ImpliedSchemaNode *o);

		void markSubtreeValue() const;
		void markSubtreeResult() const;
		void markRoot() const;

		ImpliedSchemaNode::Vector returnPaths;
	};
private:
	virtual void resetInternal();

	virtual void optimize(XQQuery *query);
	virtual ASTNode *optimize(ASTNode *item);

	PathResult generate(ASTNode *item);
	PathResult generateParenthesizedExpr(XQParenthesizedExpr *item);
	PathResult generateFunction(XQFunction *item);
	PathResult generateOperator(XQOperator *item);
	PathResult generateNav(XQNav *item);
	PathResult generateStep(XQStep *item);
	PathResult generateVariable(XQVariable *item);
	PathResult generateIf(XQIf *item);
	PathResult generateInstanceOf(XQInstanceOf *item);
	PathResult generateCastableAs(XQCastableAs *item);
	PathResult generateReturn(XQReturn *item);
	PathResult generateQuantified(XQQuantified *item);
	PathResult generateTypeswitch(XQTypeswitch *item);
	PathResult generateDOMConstructor(XQDOMConstructor *item);
	PathResult generateNameExpression(XQNameExpression *item);
	PathResult generateContentSequence(XQContentSequence *item);
	PathResult generateDirectName(XQDirectName *item);
	PathResult generateUserFunction(XQUserFunction::Instance *item);
	PathResult generateContextItem(XQContextItem *item);
	PathResult generateLiteral(XQLiteral *item);
	PathResult generateSequence(XQSequence *item);
	PathResult generateCastAs(XQCastAs *item);
	PathResult generateTreatAs(XQTreatAs *item);
	PathResult generateOrderingChange(XQOrderingChange *item);
	PathResult generateAtomize(XQAtomize *item);
	PathResult generatePromoteUntyped(XQPromoteUntyped *item);
	PathResult generatePromoteNumeric(XQPromoteNumeric *item);
	PathResult generatePromoteAnyURI(XQPromoteAnyURI *item);
	PathResult generateDocumentOrder(XQDocumentOrder *item);
	PathResult generatePredicate(XQPredicate *item);
	PathResult generateValidate(XQValidate *item);

	PathResult generateUDelete(UDelete *item);
	PathResult generateURename(URename *item);
	PathResult generateUReplace(UReplace *item);
	PathResult generateUReplaceValueOf(UReplaceValueOf *item);
	PathResult generateUInsertAsFirst(UInsertAsFirst *item);
	PathResult generateUInsertAsLast(UInsertAsLast *item);
	PathResult generateUInsertInto(UInsertInto *item);
	PathResult generateUInsertAfter(UInsertAfter *item);
	PathResult generateUInsertBefore(UInsertBefore *item);
	PathResult generateUTransform(UTransform *item);
	PathResult generateUApplyUpdates(UApplyUpdates *item);

	void generateTupleNode(TupleNode *item);
	void generateForTuple(ForTuple *item);
	void generateLetTuple(LetTuple *item);
	void generateWhereTuple(WhereTuple *item);
	void generateOrderByTuple(OrderByTuple *item);

	void generateGlobalVar(XQGlobalVariable *item);
	void generateFunctionDef(XQUserFunction *item);

	PathResult generateStep(Join::Type join, const DbXmlNodeTest *step,
		ASTNode *item);
	void generateBuiltInStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node,
				 PathResult &result);
	void generateParentStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node,
				PathResult &result);
	void generateSelfStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node,
			      PathResult &result);
	void generateDescendantOrSelfStep(ImpliedSchemaNode *target,
					  ImpliedSchemaNode &node,
					  PathResult &result);
	void generateAncestorStep(ImpliedSchemaNode *target,
				  ImpliedSchemaNode &node, PathResult &result);
	void generateAncestorOrSelfStep(ImpliedSchemaNode *target,
					ImpliedSchemaNode &node,
					PathResult &result);
	void generateFollowingStep(ImpliedSchemaNode *target,
				   ImpliedSchemaNode &node, PathResult &result);
	void generatePrecedingStep(ImpliedSchemaNode *target,
				   ImpliedSchemaNode &node, PathResult &result);
	void generateSiblingStep(ImpliedSchemaNode *target, ImpliedSchemaNode &node,
				 PathResult &result);

	void generateComparison(ImpliedSchemaNode::Type type, bool generalComp,
		VectorOfASTNodes &args, PathResult &result, ASTNode *item);
	void generateSubstring(ImpliedSchemaNode::Type type, VectorOfASTNodes &args,
		PathResult &result, ASTNode *item);
	void generateLookup(ASTNode *item, QueryPlanRoot *qpr, ImpliedSchemaNode::Type type,
		const char *child, const char *parent, PathResult &result);

	void createAnyNodeResult(PathResult &result);

	const PathResult &getCurrentContext() const;
	void setCurrentContext(const PathResult &value);
	void setVariable(const XMLCh *uri, const XMLCh *name,
		const PathResult &value);

	void addSecondaryOp(const PathResult &result);
	PathResult copyNodes(const PathResult &r);

	DynamicContext *xpc_;

	std::set<const ASTNode*> userFunctionStack_;

	typedef VarHashEntry<PathResult> VarStoreRef;
	typedef VariableStoreTemplate<PathResult> VarStore;

	XPath2MemoryManagerImpl memMgr_;
	VarStore varStore_; ///< Memory owned by memMgr_

	ImpliedSchemaNode::Vector roots_;
};

/// Adds document projection schemas so that any node that a DLS+ index points to is present
class ProjectionSchemaAdjuster : public NodeVisitingOptimizer
{
public:
	ProjectionSchemaAdjuster(DynamicContext *xpc, Optimizer *parent = 0)
		: NodeVisitingOptimizer(parent), xpc_(xpc) {}

	const ImpliedSchemaNode::Vector & getRoots() const { return roots_; }

protected:
	virtual void resetInternal() {}

	virtual QueryPlan *optimizePresence(PresenceQP *item);
	virtual QueryPlan *optimizeValue(ValueQP *item);
	virtual QueryPlan *optimizeRange(RangeQP *item);

	void addSchemaForIndex(PresenceQP *item);

	DynamicContext *xpc_;

	ImpliedSchemaNode::Vector roots_;
};

}

#endif
