//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: NodeVisitingOptimizer.hpp,v 1.38 2008/01/08 21:54:24 bostic Exp $
//

#ifndef __NODEVISITINGOPTIMIZER_HPP
#define	__NODEVISITINGOPTIMIZER_HPP

#include <xqilla/optimizer/ASTVisitor.hpp>

namespace DbXml
{

class QueryPlanToAST;
class DbXmlNodeCheck;
class DbXmlLastStepCheck;
class Map;
class DbXmlPredicate;

class QueryPlan;
class UnionQP;
class IntersectQP;
class PathsQP;
class PresenceQP;
class ValueQP;
class RangeQP;
class EmptyQP;
class SequentialScanQP;
class ValueFilterQP;
class PredicateFilterQP;
class NumericPredicateFilterQP;
class NodePredicateFilterQP;
class NegativeNodePredicateFilterQP;
class ASTToQueryPlan;
class DescendantJoinQP;
class DescendantOrSelfJoinQP;
class AncestorJoinQP;
class AncestorOrSelfJoinQP;
class AttributeJoinQP;
class ChildJoinQP;
class AttributeOrChildJoinQP;
class ParentJoinQP;
class ParentOfAttributeJoinQP;
class ParentOfChildJoinQP;
class DecisionPointQP;
class DecisionPointEndQP;
class ExceptQP;
class ContextNodeQP;
class CollectionQP;
class DocQP;
class BufferQP;
class BufferReferenceQP;
class LevelFilterQP;
class StepQP;
class VariableQP;
class DecisionPointSource;
class QueryPlanDPSource;

class NodeVisitingOptimizer : public ASTVisitor
{
public:
	NodeVisitingOptimizer(Optimizer *parent = 0)
		: ASTVisitor(parent) {}

protected:
	virtual void resetInternal() = 0;

	virtual void optimize(XQQuery *query);
	virtual ASTNode *optimize(ASTNode *item);

	virtual ASTNode *optimizeQueryPlanToAST(QueryPlanToAST *item);
	virtual ASTNode *optimizeDbXmlNodeCheck(DbXmlNodeCheck *item);
	virtual ASTNode *optimizeDbXmlLastStepCheck(DbXmlLastStepCheck *item);
	virtual ASTNode *optimizeMap(Map *item);
	virtual ASTNode *optimizeDbXmlPredicate(DbXmlPredicate *item);

	virtual QueryPlan *optimizeQP(QueryPlan *item);
	virtual QueryPlan *optimizeUnion(UnionQP *item);
	virtual QueryPlan *optimizeIntersect(IntersectQP *item);
	virtual QueryPlan *optimizePaths(PathsQP *item);
	virtual QueryPlan *optimizePresence(PresenceQP *item);
	virtual QueryPlan *optimizeValue(ValueQP *item);
	virtual QueryPlan *optimizeRange(RangeQP *item);
	virtual QueryPlan *optimizeEmpty(EmptyQP *item);
	virtual QueryPlan *optimizeSequentialScan(SequentialScanQP *item);
	virtual QueryPlan *optimizeValueFilter(ValueFilterQP *item);
	virtual QueryPlan *optimizePredicateFilter(PredicateFilterQP *item);
	virtual QueryPlan *optimizeNumericPredicateFilter(NumericPredicateFilterQP *item);
	virtual QueryPlan *optimizeNodePredicateFilter(NodePredicateFilterQP *item);
	virtual QueryPlan *optimizeNegativeNodePredicateFilter(NegativeNodePredicateFilterQP *item);
	virtual QueryPlan *optimizeASTToQueryPlan(ASTToQueryPlan *item);
	virtual QueryPlan *optimizeDescendantJoin(DescendantJoinQP *item);
	virtual QueryPlan *optimizeDescendantOrSelfJoin(DescendantOrSelfJoinQP *item);
	virtual QueryPlan *optimizeAncestorJoin(AncestorJoinQP *item);
	virtual QueryPlan *optimizeAncestorOrSelfJoin(AncestorOrSelfJoinQP *item);
	virtual QueryPlan *optimizeAttributeJoin(AttributeJoinQP *item);
	virtual QueryPlan *optimizeChildJoin(ChildJoinQP *item);
	virtual QueryPlan *optimizeAttributeOrChildJoin(AttributeOrChildJoinQP *item);
	virtual QueryPlan *optimizeParentJoin(ParentJoinQP *item);
	virtual QueryPlan *optimizeParentOfAttributeJoin(ParentOfAttributeJoinQP *item);
	virtual QueryPlan *optimizeParentOfChildJoin(ParentOfChildJoinQP *item);
	virtual QueryPlan *optimizeDecisionPoint(DecisionPointQP *item);
	virtual QueryPlan *optimizeDecisionPointEnd(DecisionPointEndQP *item);
	virtual QueryPlan *optimizeExcept(ExceptQP *item);
	virtual QueryPlan *optimizeContextNode(ContextNodeQP *item);
	virtual QueryPlan *optimizeCollection(CollectionQP *item);
	virtual QueryPlan *optimizeDoc(DocQP *item);
	virtual QueryPlan *optimizeBuffer(BufferQP *item);
	virtual QueryPlan *optimizeBufferReference(BufferReferenceQP *item);
	virtual QueryPlan *optimizeLevelFilter(LevelFilterQP *item);
	virtual QueryPlan *optimizeStepQP(StepQP *item);
	virtual QueryPlan *optimizeVariableQP(VariableQP *item);

	virtual DecisionPointSource *optimizeDPS(DecisionPointSource *item);
	virtual DecisionPointSource *optimizeQueryPlanDPSource(QueryPlanDPSource *item);
};

}

#endif
