//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: ImpliedSchemaNode.cpp,v 1.12 2008/01/08 21:54:25 bostic Exp $
//

#include "../DbXmlInternal.hpp"
#include "ImpliedSchemaNode.hpp"
#include "../UTF8.hpp"
#include "../Value.hpp"
#include "../dataItem/DbXmlNodeTest.hpp"
#include <xqilla/utils/XPath2Utils.hpp>
#include <xqilla/ast/XQCastAs.hpp>
#include <sstream>

#if defined(XERCES_HAS_CPP_NAMESPACE)
XERCES_CPP_NAMESPACE_USE
#endif

using namespace DbXml;
using namespace std;

ImpliedSchemaNode::ImpliedSchemaNode(Type type, bool generalComp, Syntax::Type syntaxType, const ASTNode *value,
	MemoryManager *mm)
	: type_(type),
	  nodeTest_(0),
	  generalComp_(generalComp),
	  syntaxType_(syntaxType),
	  astnode_(value),
	  utf8URIName_(0),
	  parent_(0),
	  nextSibling_(0),
	  prevSibling_(0),
	  firstChild_(0),
	  lastChild_(0),
	  qpr_(0),
	  memMgr_(mm)
{
}

ImpliedSchemaNode::ImpliedSchemaNode(const DbXmlNodeTest *nodeTest, Type type, MemoryManager *mm)
	: type_(type),
	  nodeTest_(nodeTest),
	  generalComp_(false),
	  syntaxType_(Syntax::NONE),
	  astnode_(0),
	  utf8URIName_(0),
	  parent_(0),
	  nextSibling_(0),
	  prevSibling_(0),
	  firstChild_(0),
	  lastChild_(0),
	  qpr_(0),
	  memMgr_(mm)
{
}

ImpliedSchemaNode::ImpliedSchemaNode(const XQCastAs *cast, XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *mm)
	: type_(CAST),
	  nodeTest_(0),
	  generalComp_(false),
	  syntaxType_(Syntax::NONE),
	  astnode_(cast),
	  utf8URIName_(0),
	  parent_(0),
	  nextSibling_(0),
	  prevSibling_(0),
	  firstChild_(0),
	  lastChild_(0),
	  qpr_(0),
	  memMgr_(mm)
{
}

ImpliedSchemaNode::~ImpliedSchemaNode()
{
	// We don't need to do this now we're memory managed

	//   ImpliedSchemaNode *child = firstChild_;
	//   ImpliedSchemaNode *next;
	//   while(child) {
	//     next = child->nextSibling_;
	//     delete child;
	//     child = next;
	//   }
	//   if(utf8URIName_) delete utf8URIName_;
}

const XQCastAs *ImpliedSchemaNode::getCast() const
{
	return type_ == CAST ? (XQCastAs*)astnode_ : 0;
}

bool ImpliedSchemaNode::isWildcardURI() const
{
	if(nodeTest_ == 0) return true;

	return nodeTest_->getItemType() || nodeTest_->getNamespaceWildcard();
}

bool ImpliedSchemaNode::isWildcardName() const
{
	if(nodeTest_ == 0) return true;

	return nodeTest_->getItemType() || nodeTest_->getNameWildcard();
}

bool ImpliedSchemaNode::isWildcardNodeType() const
{
	if(nodeTest_ == 0) return true;

	return nodeTest_->getItemType() || nodeTest_->getTypeWildcard() ||
		(nodeTest_->isNodeTypeSet() && nodeTest_->getNodeType() != Node::element_string);
}

bool ImpliedSchemaNode::isWildcard() const
{
	if(nodeTest_ == 0 || nodeTest_->getItemType()) return true;

	return nodeTest_->getNamespaceWildcard() || nodeTest_->getNameWildcard() || nodeTest_->getTypeWildcard() ||
		(type_ != ATTRIBUTE && type_ != DESCENDANT_ATTR && type_ != METADATA && nodeTest_->getNodeType() != Node::element_string);
}

bool ImpliedSchemaNode::isSuitableForIndex() const
{
	if(nodeTest_ == 0 || nodeTest_->getItemType()) return false;

	if(nodeTest_->getTypeWildcard() || (type_ != ATTRIBUTE && type_ != DESCENDANT_ATTR &&
		   nodeTest_->getNodeType() != Node::element_string)) return false;

	return true;
}

const XMLCh *ImpliedSchemaNode::getURI() const
{
	if(nodeTest_->getItemType() || nodeTest_->getNamespaceWildcard())
		return 0;
	return nodeTest_->getNodeUri();
}

const XMLCh *ImpliedSchemaNode::getName() const
{
	if(nodeTest_->getItemType() || nodeTest_->getNameWildcard())
		return 0;
	return nodeTest_->getNodeName();
}

ImpliedSchemaNode::Type ImpliedSchemaNode::getType() const
{
	return type_;
}

void ImpliedSchemaNode::setType(Type t)
{
	type_ = t;
}

ImpliedSchemaNode *ImpliedSchemaNode::getParent()
{
	return parent_;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getParent() const
{
	return parent_;
}

ImpliedSchemaNode *ImpliedSchemaNode::getNextSibling()
{
	return nextSibling_;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getNextSibling() const
{
	return nextSibling_;
}

ImpliedSchemaNode *ImpliedSchemaNode::getPreviousSibling()
{
	return prevSibling_;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getPreviousSibling() const
{
	return prevSibling_;
}

ImpliedSchemaNode *ImpliedSchemaNode::getFirstChild()
{
	return firstChild_;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getFirstChild() const
{
	return firstChild_;
}

ImpliedSchemaNode *ImpliedSchemaNode::getLastChild()
{
	return lastChild_;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getLastChild() const
{
	return lastChild_;
}

ImpliedSchemaNode *ImpliedSchemaNode::getBaseNode()
{
	switch(type_) {
	case ImpliedSchemaNode::ROOT:
	case ImpliedSchemaNode::METADATA:
	case ImpliedSchemaNode::ATTRIBUTE:
	case ImpliedSchemaNode::CHILD:
	case ImpliedSchemaNode::DESCENDANT:
	case ImpliedSchemaNode::DESCENDANT_ATTR: {
		return this;
	}
	case ImpliedSchemaNode::CAST:
	case ImpliedSchemaNode::EQUALS:
	case ImpliedSchemaNode::GTX:
	case ImpliedSchemaNode::GTE:
	case ImpliedSchemaNode::LTX:
	case ImpliedSchemaNode::LTE:
	case ImpliedSchemaNode::PREFIX:
	case ImpliedSchemaNode::SUFFIX:
	case ImpliedSchemaNode::SUBSTRING_CD:
	case ImpliedSchemaNode::SUBSTRING: {
		return parent_->getBaseNode();
	}
	}
	return 0;
}

ImpliedSchemaNode *ImpliedSchemaNode::getRoot()
{
	ImpliedSchemaNode *result = this;
	ImpliedSchemaNode *parent = parent_;
	while(parent) {
		result = parent;
		parent = parent->parent_;
	}
	return result;
}

const ImpliedSchemaNode *ImpliedSchemaNode::getRoot() const
{
	const ImpliedSchemaNode *result = this;
	const ImpliedSchemaNode *parent = parent_;
	while(parent) {
		result = parent;
		parent = parent->parent_;
	}
	return result;
}

ImpliedSchemaNode *ImpliedSchemaNode::appendChild(ImpliedSchemaNode *childToAdopt)
{
	if(childToAdopt->parent_ == this) return childToAdopt;

	if(childToAdopt->parent_ != 0) {
		childToAdopt->parent_->removeChild(childToAdopt, false);
	}

	// Search to see if the child exists already
	ImpliedSchemaNode *compare = firstChild_;
	for(; compare; compare = compare->nextSibling_) {
		if(compare->equals(childToAdopt)) {
			break;
		}
	}

	if(compare) {
		// Child exists already
		compare->stealChildren(childToAdopt);
		//     delete childToAdopt; - Not done, as we're memory managed now

		return compare;
	}
	else {
		// Child doesn't already exist, so add
		childToAdopt->parent_ = this;
		if(firstChild_ == 0) {
			firstChild_ = childToAdopt;
		}
		if(lastChild_) {
			lastChild_->nextSibling_ = childToAdopt;
			childToAdopt->prevSibling_ = lastChild_;
		}
		lastChild_ = childToAdopt;

		return childToAdopt;
	}
}

void ImpliedSchemaNode::removeChild(ImpliedSchemaNode *child, bool deleteChild)
{
	DBXML_ASSERT(child->parent_ == this);

	if(firstChild_ == child) {
		firstChild_ = child->nextSibling_;
	}

	if(lastChild_ == child) {
		lastChild_ = child->prevSibling_;
	}

	if(child->prevSibling_) {
		child->prevSibling_->nextSibling_ = child->nextSibling_;
	}

	if(child->nextSibling_) {
		child->nextSibling_->prevSibling_ = child->prevSibling_;
	}

	if(deleteChild) {
		//     delete child; - Not done, as we're memory managed now.
	}
	else {
		child->parent_ = 0;
		child->nextSibling_ = 0;
		child->prevSibling_ = 0;
	}
}

void ImpliedSchemaNode::stealChildren(ImpliedSchemaNode *victim)
{
	// Loop over the children of "victim", adding them to us
	for(ImpliedSchemaNode *i = victim->firstChild_; i != 0; i = victim->firstChild_) {
		appendChild(i); // Removes it from "victim"
	}
}

void ImpliedSchemaNode::markSubtreeValue()
{
	appendChild(new (memMgr_) ImpliedSchemaNode(0, DESCENDANT, memMgr_));
}

void ImpliedSchemaNode::markSubtreeResult()
{
	appendChild(new (memMgr_) ImpliedSchemaNode(0, DESCENDANT_ATTR, memMgr_));
	appendChild(new (memMgr_) ImpliedSchemaNode(0, DESCENDANT, memMgr_));
}

bool ImpliedSchemaNode::equals(const ImpliedSchemaNode *o) const
{
	return isWildcardURI() == o->isWildcardURI() &&
		isWildcardName() == o->isWildcardName() &&
		isWildcardNodeType() == o->isWildcardNodeType() &&
		type_ == o->type_ &&
		generalComp_ == o->generalComp_ &&
		syntaxType_ == o->syntaxType_ &&
		astnode_ == o->astnode_ &&
		(isWildcardURI() || XPath2Utils::equals(getURI(), o->getURI())) &&
		(isWildcardName() || XPath2Utils::equals(getName(), o->getName()));
}

bool ImpliedSchemaNode::isSubsetOf(const ImpliedSchemaNode *node) const
{
	return DbXmlNodeTest::isSubsetOf(nodeTest_, node->nodeTest_);
}

ImpliedSchemaNode *ImpliedSchemaNode::copy(MemoryManager *mm) const
{
	if(mm == 0) {
		mm = memMgr_;
	}

	ImpliedSchemaNode *result = new (mm) ImpliedSchemaNode(nodeTest_, type_, mm);

	ImpliedSchemaNode *child;
	for(child = firstChild_; child; child = child->nextSibling_) {
		result->appendChild(child->copy(mm));
	}

	return result;
}

static string getIndent(int level)
{
	ostringstream s;

	for(int i = level; i != 0; --i) {
		s << "  ";
	}

	return s.str();
}

static string typeToString(ImpliedSchemaNode::Type type) {
	switch(type) {
	case ImpliedSchemaNode::ATTRIBUTE: {
		return "attribute";
		break;
	}
	case ImpliedSchemaNode::CHILD: {
		return "child";
		break;
	}
	case ImpliedSchemaNode::DESCENDANT: {
		return "descendant";
		break;
	}
	case ImpliedSchemaNode::DESCENDANT_ATTR: {
		return "descendant-attr";
		break;
	}
	case ImpliedSchemaNode::ROOT: {
		return "root";
		break;
	}
	case ImpliedSchemaNode::METADATA: {
		return "metadata";
		break;
	}
	case ImpliedSchemaNode::CAST: {
		return "cast";
		break;
	}
	case ImpliedSchemaNode::EQUALS: {
		return "equals";
		break;
	}
	case ImpliedSchemaNode::GTX: {
		return "gtx";
		break;
	}
	case ImpliedSchemaNode::GTE: {
		return "gte";
		break;
	}
	case ImpliedSchemaNode::LTX: {
		return "ltx";
		break;
	}
	case ImpliedSchemaNode::LTE: {
		return "lte";
		break;
	}
	case ImpliedSchemaNode::PREFIX: {
		return "prefix";
		break;
	}
	case ImpliedSchemaNode::SUFFIX: {
		return "suffix";
		break;
	}
	case ImpliedSchemaNode::SUBSTRING_CD: {
		return "substring_cd";
		break;
	}
	case ImpliedSchemaNode::SUBSTRING: {
		return "substring";
		break;
	}
	}
	return "UNKNOWN";
}

string ImpliedSchemaNode::toString(int level) const
{
	ostringstream s;
	string in(getIndent(level));
	string type(typeToString(type_));

	s << in << "<" << type;

	switch(type_) {
	case ImpliedSchemaNode::ROOT:
	case ImpliedSchemaNode::METADATA:
	case ImpliedSchemaNode::ATTRIBUTE:
	case ImpliedSchemaNode::CHILD:
	case ImpliedSchemaNode::DESCENDANT:
	case ImpliedSchemaNode::DESCENDANT_ATTR:
		if(isWildcardURI()) {
			s << " uri=\"*\"";
		} else if(nodeTest_->getNodeUri8() == 0) {
			s << " uri=\"\"";
		} else {
			s << " uri=\"" << (char*)nodeTest_->getNodeUri8() << "\"";
		}
		if(isWildcardName()) {
			s << " name=\"*\"";
		} else if(nodeTest_->getNodeName8() == 0) {
			s << " name=\"\"";
		} else {
			s << " name=\"" << (char*)nodeTest_->getNodeName8() << "\"";
		}
		if(isWildcardNodeType()) {
			s << " nodeType=\"*\"";
		}
		break;
	case ImpliedSchemaNode::CAST:
		s << " uri=\"" << XMLChToUTF8(getCast()->getSequenceType()->getTypeURI(0)).str() << "\"";
		s << " name=\"" << XMLChToUTF8(getCast()->getSequenceType()->getConstrainingType()->getName()).str() << "\"";
		break;
	default:
		break;
	}

	if(firstChild_) {
		s << ">" << endl;

		ImpliedSchemaNode *child = firstChild_;
		while(child) {
			s << child->toString(level + 1);
			child = child->nextSibling_;
		}

		s << in << "</" << type << ">" << endl;
	}
	else {
		s << "/>" << endl;
	}

	return s.str();
}

string ImpliedSchemaNode::getPath() const
{
	ostringstream s;

	if(parent_) {
		s << parent_->getPath();
	}

	if(type_ == CHILD || type_ == ATTRIBUTE || type_ == DESCENDANT || type_ == DESCENDANT_ATTR) {
		s << "/";
	}

	s << getStepName();

	return s.str();
}

string ImpliedSchemaNode::getStepName() const
{
	ostringstream s;

	switch(type_) {
	case ROOT: {
		s << "root()";
		break;
	}
	case CAST:
	case GTX:
	case GTE:
	case LTX:
	case LTE:
	case EQUALS:
	case PREFIX:
	case SUFFIX:
	case SUBSTRING_CD:
	case SUBSTRING: {
		switch(type_) {
		case CAST: s << " [cast] "; break;
		case GTX: s << " [gt] "; break;
		case GTE: s << " [ge] "; break;
		case LTX: s << " [lt] "; break;
		case LTE: s << " [le] "; break;
		case EQUALS: s << " [eq] "; break;
		case PREFIX: s << " [prefix] "; break;
		case SUFFIX: s << " [suffix] "; break;
		case SUBSTRING: s << " [substring] "; break;
		case SUBSTRING_CD: s << " [substring_cd] "; break;
		default: break;
		}
		break;
	}
	default: {
		if(type_ == ATTRIBUTE) {
			s << "@";
		}
		else if(type_ != CHILD) {
			s << typeToString(type_) << "::";
		}

		if(isWildcardNodeType() && type_ != ATTRIBUTE && type_ != DESCENDANT_ATTR) {
			s << "node()";
		}
		else {
			s << getUriName();
		}
		break;
	}
	}

	return s.str();
}

const char *ImpliedSchemaNode::getURI8() const
{
	if(nodeTest_->getItemType() || nodeTest_->getNamespaceWildcard())
		return 0;
	return (char*)nodeTest_->getNodeUri8();
}

const char *ImpliedSchemaNode::getName8() const
{
	if(nodeTest_->getItemType() || nodeTest_->getNameWildcard())
		return 0;
	return (char*)nodeTest_->getNodeName8();
}

// uriname must be name:uri, to match
// Name.cpp usage
const char *ImpliedSchemaNode::getUriName() const
{
	if(utf8URIName_ == 0) {
		ostringstream s;

		if(isWildcardURI() && isWildcardName()) {
			s << "*";
		} else {
			if(isWildcardName()) {
				s << "*";
			} else {
				s << (char*)nodeTest_->getNodeName8();
			}
			if(isWildcardURI()) {
				s << ":*";
			} else if(nodeTest_->getNodeUri8() != 0) {
				s << ":" << (char*)nodeTest_->getNodeUri8();
			}
		}

		const_cast<ImpliedSchemaNode*>(this)->utf8URIName_ = XMLString::replicate(s.str().c_str(), memMgr_);
	}
	return utf8URIName_;
}
