//
// See the file LICENSE for redistribution information.
//
// Copyright (c) 2002,2008 Oracle.  All rights reserved.
//
// $Id: ImpliedSchemaNode.hpp,v 1.11 2008/01/08 21:54:25 bostic Exp $
//

#ifndef __IMPLIEDSCHEMANODE_HPP
#define	__IMPLIEDSCHEMANODE_HPP

#include <vector>
#include <string>

#include <xqilla/framework/XPath2MemoryManager.hpp>

#include "Syntax.hpp"

class ASTNode;
class XQCastAs;

namespace DbXml
{

class QueryPlanRoot;
class DbXmlNodeTest;

class ImpliedSchemaNode
{
public:
	typedef enum {
		ATTRIBUTE,
		CHILD,
		DESCENDANT,
		DESCENDANT_ATTR, ///< Only used by ProjectionSchemaAdjuster
		ROOT,
		METADATA,
		CAST,
		EQUALS,
		LTX,
		LTE,
		GTX,
		GTE,
		PREFIX,
		SUBSTRING,
		SUBSTRING_CD, ///< Case/diacritic insensitive
		SUFFIX
	} Type;

	typedef std::vector<ImpliedSchemaNode*> Vector;
	typedef std::vector<ImpliedSchemaNode*, XQillaAllocator<ImpliedSchemaNode*> > MVector;


	ImpliedSchemaNode(Type type, bool generalComp, Syntax::Type syntaxType, const ASTNode *value,
		XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *mm);
	ImpliedSchemaNode(const DbXmlNodeTest *nodeTest, Type type, XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *mm);
	ImpliedSchemaNode(const XQCastAs *cast, XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *mm);
	~ImpliedSchemaNode();

	const QueryPlanRoot *getQueryPlanRoot() const { return qpr_; }
	void setQueryPlanRoot(QueryPlanRoot *qpr) { qpr_ = qpr; }

	bool isWildcardURI() const;
	bool isWildcardName() const;
	bool isWildcardNodeType() const;

	bool isWildcard() const;
	bool isSuitableForIndex() const;

	const XMLCh *getURI() const;
	const XMLCh *getName() const;

	const DbXmlNodeTest *getNodeTest() const { return nodeTest_; }
	const XQCastAs *getCast() const;

	Type getType() const;
	void setType(Type t);

	bool getGeneralComp() const { return generalComp_; }
	void setGeneralComp(bool v) { generalComp_ = v; }
	Syntax::Type getSyntaxType() const { return syntaxType_; }
	void setSyntaxType(Syntax::Type st) { syntaxType_ = st; }
	const ASTNode *getASTNode() const { return astnode_; }
	void setASTNode(const ASTNode *value) { astnode_ = value; }

	ImpliedSchemaNode *getParent();
	const ImpliedSchemaNode *getParent() const;
	ImpliedSchemaNode *getNextSibling();
	const ImpliedSchemaNode *getNextSibling() const;
	ImpliedSchemaNode *getPreviousSibling();
	const ImpliedSchemaNode *getPreviousSibling() const;
	ImpliedSchemaNode *getFirstChild();
	const ImpliedSchemaNode *getFirstChild() const;
	ImpliedSchemaNode *getLastChild();
	const ImpliedSchemaNode *getLastChild() const;

	ImpliedSchemaNode *getRoot();
	const ImpliedSchemaNode *getRoot() const;

	ImpliedSchemaNode *getBaseNode();

	ImpliedSchemaNode *appendChild(ImpliedSchemaNode *childToAdopt);
	void removeChild(ImpliedSchemaNode *child, bool deleteChild = true);
	void stealChildren(ImpliedSchemaNode *victim);
	void markSubtreeValue();
	void markSubtreeResult();

	std::string toString(int level = 0) const;
	std::string getPath() const;
	std::string getStepName() const;

	const char *getURI8() const;
	const char *getName8() const;
	/// Lazily calculated
	const char *getUriName() const;

	bool equals(const ImpliedSchemaNode *o) const;
	bool isSubsetOf(const ImpliedSchemaNode *node) const;

	/// Performs a deep copy of the node and it's children
	ImpliedSchemaNode *copy(XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *mm = 0) const;

private:
	ImpliedSchemaNode(const ImpliedSchemaNode &);
	ImpliedSchemaNode &operator=(const ImpliedSchemaNode &);

	Type type_;

	const DbXmlNodeTest *nodeTest_;

	bool generalComp_;
	Syntax::Type syntaxType_;
	const ASTNode *astnode_;

	char *utf8URIName_;

	ImpliedSchemaNode *parent_;
	ImpliedSchemaNode *nextSibling_;
	ImpliedSchemaNode *prevSibling_;
	ImpliedSchemaNode *firstChild_;
	ImpliedSchemaNode *lastChild_;

	QueryPlanRoot *qpr_;

	XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *memMgr_;
};

}

#endif
